from typing import Any

log: Any

class DataWatch:
    def __init__(self, client, path, func=..., *args, **kwargs) -> None: ...
    def __call__(self, func): ...

class ChildrenWatch:
    def __init__(self, client, path, func=..., allow_session_lost=..., send_event=...) -> None: ...
    def __call__(self, func): ...

class PatientChildrenWatch:
    client: Any
    path: Any
    children: Any
    time_boundary: Any
    children_changed: Any
    def __init__(self, client, path, time_boundary=...) -> None: ...
    asy: Any
    def start(self): ...



from types import FrameType
from typing import Any, Callable, Tuple, Union

SIG_DFL: int = ...
SIG_IGN: int = ...

ITIMER_REAL: int = ...
ITIMER_VIRTUAL: int = ...
ITIMER_PROF: int = ...

NSIG: int = ...

SIGABRT: int = ...
SIGALRM: int = ...
SIGBREAK: int = ...  # Windows
SIGBUS: int = ...
SIGCHLD: int = ...
SIGCLD: int = ...
SIGCONT: int = ...
SIGEMT: int = ...
SIGFPE: int = ...
SIGHUP: int = ...
SIGILL: int = ...
SIGINFO: int = ...
SIGINT: int = ...
SIGIO: int = ...
SIGIOT: int = ...
SIGKILL: int = ...
SIGPIPE: int = ...
SIGPOLL: int = ...
SIGPROF: int = ...
SIGPWR: int = ...
SIGQUIT: int = ...
SIGRTMAX: int = ...
SIGRTMIN: int = ...
SIGSEGV: int = ...
SIGSTOP: int = ...
SIGSYS: int = ...
SIGTERM: int = ...
SIGTRAP: int = ...
SIGTSTP: int = ...
SIGTTIN: int = ...
SIGTTOU: int = ...
SIGURG: int = ...
SIGUSR1: int = ...
SIGUSR2: int = ...
SIGVTALRM: int = ...
SIGWINCH: int = ...
SIGXCPU: int = ...
SIGXFSZ: int = ...

# Windows
CTRL_C_EVENT: int = ...
CTRL_BREAK_EVENT: int = ...

class ItimerError(IOError): ...

_HANDLER = Union[Callable[[int, FrameType], None], int, None]

def alarm(time: int) -> int: ...
def getsignal(signalnum: int) -> _HANDLER: ...
def pause() -> None: ...
def setitimer(which: int, seconds: float, interval: float = ...) -> Tuple[float, float]: ...
def getitimer(which: int) -> Tuple[float, float]: ...
def set_wakeup_fd(fd: int) -> int: ...
def siginterrupt(signalnum: int, flag: bool) -> None: ...
def signal(signalnum: int, handler: _HANDLER) -> _HANDLER: ...
def default_int_handler(signum: int, frame: FrameType) -> None: ...



import sys
from typing import AnyStr, Sequence, Text, Union

if sys.version_info >= (3, 0):
    def commonprefix(m: Sequence[str]) -> str: ...

else:
    def commonprefix(m: Sequence[AnyStr]) -> AnyStr: ...

if sys.version_info >= (3, 6):
    from builtins import _PathLike
    def exists(path: Union[AnyStr, _PathLike[AnyStr]]) -> bool: ...

else:
    def exists(path: Text) -> bool: ...

def isfile(path: Text) -> bool: ...
def isdir(s: Text) -> bool: ...
def getsize(filename: Text) -> int: ...
def getmtime(filename: Text) -> float: ...
def getatime(filename: Text) -> float: ...
def getctime(filename: Text) -> float: ...

if sys.version_info >= (3, 4):
    def samestat(s1: str, s2: str) -> int: ...
    def samefile(f1: str, f2: str) -> int: ...
    def sameopenfile(fp1: str, fp2: str) -> int: ...



from typing import Any, Callable, Container, Dict, Iterable, List, Mapping, NamedTuple, Optional, Sequence, Union

PRESERVE_WHITESPACE: bool
WIDE_CHARS_MODE: bool
tabulate_formats: List[str]

class Line(NamedTuple):
    begin: str
    hline: str
    sep: str
    end: str

class DataRow(NamedTuple):
    begin: str
    sep: str
    end: str

_TableFormatLine = Union[None, Line, Callable[[List[int], List[str]], str]]
_TableFormatRow = Union[None, DataRow, Callable[[List[Any], List[int], List[str]], str]]

class TableFormat(NamedTuple):
    lineabove: _TableFormatLine
    linebelowheader: _TableFormatLine
    linebetweenrows: _TableFormatLine
    linebelow: _TableFormatLine
    headerrow: _TableFormatRow
    datarow: _TableFormatRow
    padding: int
    with_header_hide: Optional[Container[str]]

def simple_separated_format(separator: str) -> TableFormat: ...
def tabulate(
    tabular_data: Union[Mapping[str, Iterable[Any]], Iterable[Iterable[Any]]],
    headers: Union[str, Dict[str, str], Sequence[str]] = ...,
    tablefmt: Union[str, TableFormat] = ...,
    floatfmt: Union[str, Iterable[str]] = ...,
    numalign: Optional[str] = ...,
    stralign: Optional[str] = ...,
    missingval: Union[str, Iterable[str]] = ...,
    showindex: Union[str, bool, Iterable[Any]] = ...,
    disable_numparse: Union[bool, Iterable[int]] = ...,
    colalign: Optional[Iterable[Optional[str]]] = ...,
) -> str: ...



import pyexpat.errors as errors
import pyexpat.model as model
from _typeshed import SupportsRead
from typing import Any, Callable, Dict, List, Optional, Text, Tuple, Union

EXPAT_VERSION: str  # undocumented
version_info: Tuple[int, int, int]  # undocumented
native_encoding: str  # undocumented
features: List[Tuple[str, int]]  # undocumented

class ExpatError(Exception):
    code: int
    lineno: int
    offset: int

error = ExpatError

XML_PARAM_ENTITY_PARSING_NEVER: int
XML_PARAM_ENTITY_PARSING_UNLESS_STANDALONE: int
XML_PARAM_ENTITY_PARSING_ALWAYS: int

_Model = Tuple[int, int, Optional[str], tuple]

class XMLParserType(object):
    def Parse(self, __data: Union[Text, bytes], __isfinal: bool = ...) -> int: ...
    def ParseFile(self, __file: SupportsRead[bytes]) -> int: ...
    def SetBase(self, __base: Text) -> None: ...
    def GetBase(self) -> Optional[str]: ...
    def GetInputContext(self) -> Optional[bytes]: ...
    def ExternalEntityParserCreate(self, __context: Optional[Text], __encoding: Text = ...) -> XMLParserType: ...
    def SetParamEntityParsing(self, __flag: int) -> int: ...
    def UseForeignDTD(self, __flag: bool = ...) -> None: ...
    buffer_size: int
    buffer_text: bool
    buffer_used: int
    namespace_prefixes: bool  # undocumented
    ordered_attributes: bool
    specified_attributes: bool
    ErrorByteIndex: int
    ErrorCode: int
    ErrorColumnNumber: int
    ErrorLineNumber: int
    CurrentByteIndex: int
    CurrentColumnNumber: int
    CurrentLineNumber: int
    XmlDeclHandler: Optional[Callable[[str, Optional[str], int], Any]]
    StartDoctypeDeclHandler: Optional[Callable[[str, Optional[str], Optional[str], bool], Any]]
    EndDoctypeDeclHandler: Optional[Callable[[], Any]]
    ElementDeclHandler: Optional[Callable[[str, _Model], Any]]
    AttlistDeclHandler: Optional[Callable[[str, str, str, Optional[str], bool], Any]]
    StartElementHandler: Optional[
        Union[
            Callable[[str, Dict[str, str]], Any],
            Callable[[str, List[str]], Any],
            Callable[[str, Union[Dict[str, str]], List[str]], Any],
        ]
    ]
    EndElementHandler: Optional[Callable[[str], Any]]
    ProcessingInstructionHandler: Optional[Callable[[str, str], Any]]
    CharacterDataHandler: Optional[Callable[[str], Any]]
    UnparsedEntityDeclHandler: Optional[Callable[[str, Optional[str], str, Optional[str], str], Any]]
    EntityDeclHandler: Optional[Callable[[str, bool, Optional[str], Optional[str], str, Optional[str], Optional[str]], Any]]
    NotationDeclHandler: Optional[Callable[[str, Optional[str], str, Optional[str]], Any]]
    StartNamespaceDeclHandler: Optional[Callable[[str, str], Any]]
    EndNamespaceDeclHandler: Optional[Callable[[str], Any]]
    CommentHandler: Optional[Callable[[str], Any]]
    StartCdataSectionHandler: Optional[Callable[[], Any]]
    EndCdataSectionHandler: Optional[Callable[[], Any]]
    DefaultHandler: Optional[Callable[[str], Any]]
    DefaultHandlerExpand: Optional[Callable[[str], Any]]
    NotStandaloneHandler: Optional[Callable[[], int]]
    ExternalEntityRefHandler: Optional[Callable[[str, Optional[str], Optional[str], Optional[str]], int]]

def ErrorString(__code: int) -> str: ...

# intern is undocumented
def ParserCreate(
    encoding: Optional[Text] = ..., namespace_separator: Optional[Text] = ..., intern: Optional[Dict[str, Any]] = ...
) -> XMLParserType: ...



from typing import Any, List, Iterable, Optional

from django.apps.config import AppConfig

def check_generic_foreign_keys(app_configs: Optional[Iterable[AppConfig]] = ..., **kwargs: Any) -> List[Any]: ...
def check_model_name_lengths(app_configs: Optional[Iterable[AppConfig]] = ..., **kwargs: Any) -> List[Any]: ...



import _random
import sys
from typing import AbstractSet, Any, Callable, Iterable, List, MutableSequence, Optional, Sequence, Tuple, TypeVar, Union

_T = TypeVar("_T")

class Random(_random.Random):
    def __init__(self, x: Any = ...) -> None: ...
    def seed(self, a: Any = ..., version: int = ...) -> None: ...
    def getstate(self) -> Tuple[Any, ...]: ...
    def setstate(self, state: Tuple[Any, ...]) -> None: ...
    def getrandbits(self, __k: int) -> int: ...
    def randrange(self, start: int, stop: Optional[int] = ..., step: int = ...) -> int: ...
    def randint(self, a: int, b: int) -> int: ...
    if sys.version_info >= (3, 9):
        def randbytes(self, n: int) -> bytes: ...
    def choice(self, seq: Sequence[_T]) -> _T: ...
    def choices(
        self,
        population: Sequence[_T],
        weights: Optional[Sequence[float]] = ...,
        *,
        cum_weights: Optional[Sequence[float]] = ...,
        k: int = ...,
    ) -> List[_T]: ...
    def shuffle(self, x: MutableSequence[Any], random: Optional[Callable[[], float]] = ...) -> None: ...
    if sys.version_info >= (3, 9):
        def sample(
            self, population: Union[Sequence[_T], AbstractSet[_T]], k: int, *, counts: Optional[Iterable[_T]] = ...
        ) -> List[_T]: ...
    else:
        def sample(self, population: Union[Sequence[_T], AbstractSet[_T]], k: int) -> List[_T]: ...
    def random(self) -> float: ...
    def uniform(self, a: float, b: float) -> float: ...
    def triangular(self, low: float = ..., high: float = ..., mode: Optional[float] = ...) -> float: ...
    def betavariate(self, alpha: float, beta: float) -> float: ...
    def expovariate(self, lambd: float) -> float: ...
    def gammavariate(self, alpha: float, beta: float) -> float: ...
    def gauss(self, mu: float, sigma: float) -> float: ...
    def lognormvariate(self, mu: float, sigma: float) -> float: ...
    def normalvariate(self, mu: float, sigma: float) -> float: ...
    def vonmisesvariate(self, mu: float, kappa: float) -> float: ...
    def paretovariate(self, alpha: float) -> float: ...
    def weibullvariate(self, alpha: float, beta: float) -> float: ...

# SystemRandom is not implemented for all OS's; good on Windows & Linux
class SystemRandom(Random): ...

# ----- random function stubs -----
def seed(a: Any = ..., version: int = ...) -> None: ...
def getstate() -> object: ...
def setstate(state: object) -> None: ...
def getrandbits(__k: int) -> int: ...
def randrange(start: int, stop: Union[None, int] = ..., step: int = ...) -> int: ...
def randint(a: int, b: int) -> int: ...

if sys.version_info >= (3, 9):
    def randbytes(n: int) -> bytes: ...

def choice(seq: Sequence[_T]) -> _T: ...
def choices(
    population: Sequence[_T],
    weights: Optional[Sequence[float]] = ...,
    *,
    cum_weights: Optional[Sequence[float]] = ...,
    k: int = ...,
) -> List[_T]: ...
def shuffle(x: MutableSequence[Any], random: Optional[Callable[[], float]] = ...) -> None: ...

if sys.version_info >= (3, 9):
    def sample(population: Union[Sequence[_T], AbstractSet[_T]], k: int, *, counts: Optional[Iterable[_T]] = ...) -> List[_T]: ...

else:
    def sample(population: Union[Sequence[_T], AbstractSet[_T]], k: int) -> List[_T]: ...

def random() -> float: ...
def uniform(a: float, b: float) -> float: ...
def triangular(low: float = ..., high: float = ..., mode: Optional[float] = ...) -> float: ...
def betavariate(alpha: float, beta: float) -> float: ...
def expovariate(lambd: float) -> float: ...
def gammavariate(alpha: float, beta: float) -> float: ...
def gauss(mu: float, sigma: float) -> float: ...
def lognormvariate(mu: float, sigma: float) -> float: ...
def normalvariate(mu: float, sigma: float) -> float: ...
def vonmisesvariate(mu: float, kappa: float) -> float: ...
def paretovariate(alpha: float) -> float: ...
def weibullvariate(alpha: float, beta: float) -> float: ...



from typing import Any, Iterator

from cryptography.x509 import GeneralName, ObjectIdentifier

class Extension:
    value: Any = ...

class GeneralNames:
    def __iter__(self) -> Iterator[GeneralName]: ...

class DistributionPoint:
    full_name: GeneralNames = ...

class CRLDistributionPoints:
    def __iter__(self) -> Iterator[DistributionPoint]: ...

class AccessDescription:
    access_method: ObjectIdentifier = ...
    access_location: GeneralName = ...

class AuthorityInformationAccess:
    def __iter__(self) -> Iterator[AccessDescription]: ...



from typing import Any, List, Iterable, Optional

from django.core.checks.messages import Error

from django.apps.config import AppConfig

def check_finders(app_configs: Optional[Iterable[AppConfig]] = ..., **kwargs: Any) -> List[Error]: ...



from typing import Any, Callable, Optional, Sequence, TypeVar

_T = TypeVar("_T")

_F = TypeVar("_F", bound=Callable[..., Any])
_RET = Callable[[_F], _F]

def lfu_cache(maxsize: int = ..., typed: bool = ...) -> _RET: ...
def lru_cache(maxsize: int = ..., typed: bool = ...) -> _RET: ...
def rr_cache(maxsize: int = ..., choice: Optional[Callable[[Sequence[_T]], _T]] = ..., typed: bool = ...) -> _RET: ...
def ttl_cache(maxsize: int = ..., ttl: float = ..., timer: float = ..., typed: bool = ...) -> _RET: ...



from typing import Any, Optional

argument_types: Any
converters: Any

def run(namespace: Optional[Any] = ..., action_prefix: str = ..., args: Optional[Any] = ...): ...
def fail(message, code: int = ...): ...
def find_actions(namespace, action_prefix): ...
def print_usage(actions): ...
def analyse_action(func): ...
def make_shell(init_func: Optional[Any] = ..., banner: Optional[Any] = ..., use_ipython: bool = ...): ...
def make_runserver(
    app_factory,
    hostname: str = ...,
    port: int = ...,
    use_reloader: bool = ...,
    use_debugger: bool = ...,
    use_evalex: bool = ...,
    threaded: bool = ...,
    processes: int = ...,
    static_files: Optional[Any] = ...,
    extra_files: Optional[Any] = ...,
    ssl_context: Optional[Any] = ...,
): ...



from typing import Any, Callable, Dict, Optional, Sequence, Type, Union

from django.forms.forms import BaseForm
from django.views.generic.base import ContextMixin, TemplateResponseMixin, View
from django.views.generic.detail import BaseDetailView, SingleObjectMixin, SingleObjectTemplateResponseMixin
from typing_extensions import Literal

from django.http import HttpRequest, HttpResponse

class FormMixin(ContextMixin):
    initial: Dict[str, Any] = ...
    form_class: Optional[Type[BaseForm]] = ...
    success_url: Optional[Union[str, Callable[..., Any]]] = ...
    prefix: Optional[str] = ...
    def get_initial(self) -> Dict[str, Any]: ...
    def get_prefix(self) -> Optional[str]: ...
    def get_form_class(self) -> Type[BaseForm]: ...
    def get_form(self, form_class: Optional[Type[BaseForm]] = ...) -> BaseForm: ...
    def get_form_kwargs(self) -> Dict[str, Any]: ...
    def get_success_url(self) -> str: ...
    def form_valid(self, form: BaseForm) -> HttpResponse: ...
    def form_invalid(self, form: BaseForm) -> HttpResponse: ...

class ModelFormMixin(FormMixin, SingleObjectMixin):
    fields: Optional[Union[Sequence[str], Literal["__all__"]]] = ...

class ProcessFormView(View):
    def get(self, request: HttpRequest, *args: str, **kwargs: Any) -> HttpResponse: ...
    def post(self, request: HttpRequest, *args: str, **kwargs: Any) -> HttpResponse: ...
    def put(self, *args: str, **kwargs: Any) -> HttpResponse: ...

class BaseFormView(FormMixin, ProcessFormView): ...
class FormView(TemplateResponseMixin, BaseFormView): ...
class BaseCreateView(ModelFormMixin, ProcessFormView): ...
class CreateView(SingleObjectTemplateResponseMixin, BaseCreateView): ...
class BaseUpdateView(ModelFormMixin, ProcessFormView): ...
class UpdateView(SingleObjectTemplateResponseMixin, BaseUpdateView): ...

class DeletionMixin:
    success_url: Optional[str] = ...
    def post(self, request: HttpRequest, *args: str, **kwargs: Any) -> HttpResponse: ...
    def delete(self, request: HttpRequest, *args: str, **kwargs: Any) -> HttpResponse: ...
    def get_success_url(self) -> str: ...

class BaseDeleteView(DeletionMixin, BaseDetailView): ...
class DeleteView(SingleObjectTemplateResponseMixin, BaseDeleteView): ...



from typing import AnyStr, Iterator, List, Union

def glob(pathname: AnyStr) -> List[AnyStr]: ...
def iglob(pathname: AnyStr) -> Iterator[AnyStr]: ...
def glob1(dirname: Union[str, unicode], pattern: AnyStr) -> List[AnyStr]: ...
def glob0(dirname: Union[str, unicode], basename: AnyStr) -> List[AnyStr]: ...
def has_magic(s: Union[str, unicode]) -> bool: ...  # undocumented



"""
@generated by mypy-protobuf.  Do not edit manually!
isort:skip_file
"""
from google.protobuf.descriptor import (
    Descriptor as google___protobuf___descriptor___Descriptor,
    EnumDescriptor as google___protobuf___descriptor___EnumDescriptor,
    FileDescriptor as google___protobuf___descriptor___FileDescriptor,
)

from google.protobuf.internal.containers import (
    RepeatedCompositeFieldContainer as google___protobuf___internal___containers___RepeatedCompositeFieldContainer,
    RepeatedScalarFieldContainer as google___protobuf___internal___containers___RepeatedScalarFieldContainer,
)

from google.protobuf.internal.enum_type_wrapper import (
    _EnumTypeWrapper as google___protobuf___internal___enum_type_wrapper____EnumTypeWrapper,
)

from google.protobuf.message import (
    Message as google___protobuf___message___Message,
)

from typing import (
    Iterable as typing___Iterable,
    NewType as typing___NewType,
    Optional as typing___Optional,
    Text as typing___Text,
    cast as typing___cast,
)

from typing_extensions import (
    Literal as typing_extensions___Literal,
)


builtin___bool = bool
builtin___bytes = bytes
builtin___float = float
builtin___int = int


DESCRIPTOR: google___protobuf___descriptor___FileDescriptor = ...

class FileDescriptorSet(google___protobuf___message___Message):
    DESCRIPTOR: google___protobuf___descriptor___Descriptor = ...

    @property
    def file(self) -> google___protobuf___internal___containers___RepeatedCompositeFieldContainer[type___FileDescriptorProto]: ...

    def __init__(self,
        *,
        file : typing___Optional[typing___Iterable[type___FileDescriptorProto]] = None,
        ) -> None: ...
    def ClearField(self, field_name: typing_extensions___Literal[u"file",b"file"]) -> None: ...
type___FileDescriptorSet = FileDescriptorSet

class FileDescriptorProto(google___protobuf___message___Message):
    DESCRIPTOR: google___protobuf___descriptor___Descriptor = ...
    name: typing___Text = ...
    package: typing___Text = ...
    dependency: google___protobuf___internal___containers___RepeatedScalarFieldContainer[typing___Text] = ...
    public_dependency: google___protobuf___internal___containers___RepeatedScalarFieldContainer[builtin___int] = ...
    weak_dependency: google___protobuf___internal___containers___RepeatedScalarFieldContainer[builtin___int] = ...
    syntax: typing___Text = ...

    @property
    def message_type(self) -> google___protobuf___internal___containers___RepeatedCompositeFieldContainer[type___DescriptorProto]: ...

    @property
    def enum_type(self) -> google___protobuf___internal___containers___RepeatedCompositeFieldContainer[type___EnumDescriptorProto]: ...

    @property
    def service(self) -> google___protobuf___internal___containers___RepeatedCompositeFieldContainer[type___ServiceDescriptorProto]: ...

    @property
    def extension(self) -> google___protobuf___internal___containers___RepeatedCompositeFieldContainer[type___FieldDescriptorProto]: ...

    @property
    def options(self) -> type___FileOptions: ...

    @property
    def source_code_info(self) -> type___SourceCodeInfo: ...

    def __init__(self,
        *,
        name : typing___Optional[typing___Text] = None,
        package : typing___Optional[typing___Text] = None,
        dependency : typing___Optional[typing___Iterable[typing___Text]] = None,
        public_dependency : typing___Optional[typing___Iterable[builtin___int]] = None,
        weak_dependency : typing___Optional[typing___Iterable[builtin___int]] = None,
        message_type : typing___Optional[typing___Iterable[type___DescriptorProto]] = None,
        enum_type : typing___Optional[typing___Iterable[type___EnumDescriptorProto]] = None,
        service : typing___Optional[typing___Iterable[type___ServiceDescriptorProto]] = None,
        extension : typing___Optional[typing___Iterable[type___FieldDescriptorProto]] = None,
        options : typing___Optional[type___FileOptions] = None,
        source_code_info : typing___Optional[type___SourceCodeInfo] = None,
        syntax : typing___Optional[typing___Text] = None,
        ) -> None: ...
    def HasField(self, field_name: typing_extensions___Literal[u"name",b"name",u"options",b"options",u"package",b"package",u"source_code_info",b"source_code_info",u"syntax",b"syntax"]) -> builtin___bool: ...
    def ClearField(self, field_name: typing_extensions___Literal[u"dependency",b"dependency",u"enum_type",b"enum_type",u"extension",b"extension",u"message_type",b"message_type",u"name",b"name",u"options",b"options",u"package",b"package",u"public_dependency",b"public_dependency",u"service",b"service",u"source_code_info",b"source_code_info",u"syntax",b"syntax",u"weak_dependency",b"weak_dependency"]) -> None: ...
type___FileDescriptorProto = FileDescriptorProto

class DescriptorProto(google___protobuf___message___Message):
    DESCRIPTOR: google___protobuf___descriptor___Descriptor = ...
    class ExtensionRange(google___protobuf___message___Message):
        DESCRIPTOR: google___protobuf___descriptor___Descriptor = ...
        start: builtin___int = ...
        end: builtin___int = ...

        @property
        def options(self) -> type___ExtensionRangeOptions: ...

        def __init__(self,
            *,
            start : typing___Optional[builtin___int] = None,
            end : typing___Optional[builtin___int] = None,
            options : typing___Optional[type___ExtensionRangeOptions] = None,
            ) -> None: ...
        def HasField(self, field_name: typing_extensions___Literal[u"end",b"end",u"options",b"options",u"start",b"start"]) -> builtin___bool: ...
        def ClearField(self, field_name: typing_extensions___Literal[u"end",b"end",u"options",b"options",u"start",b"start"]) -> None: ...
    type___ExtensionRange = ExtensionRange

    class ReservedRange(google___protobuf___message___Message):
        DESCRIPTOR: google___protobuf___descriptor___Descriptor = ...
        start: builtin___int = ...
        end: builtin___int = ...

        def __init__(self,
            *,
            start : typing___Optional[builtin___int] = None,
            end : typing___Optional[builtin___int] = None,
            ) -> None: ...
        def HasField(self, field_name: typing_extensions___Literal[u"end",b"end",u"start",b"start"]) -> builtin___bool: ...
        def ClearField(self, field_name: typing_extensions___Literal[u"end",b"end",u"start",b"start"]) -> None: ...
    type___ReservedRange = ReservedRange

    name: typing___Text = ...
    reserved_name: google___protobuf___internal___containers___RepeatedScalarFieldContainer[typing___Text] = ...

    @property
    def field(self) -> google___protobuf___internal___containers___RepeatedCompositeFieldContainer[type___FieldDescriptorProto]: ...

    @property
    def extension(self) -> google___protobuf___internal___containers___RepeatedCompositeFieldContainer[type___FieldDescriptorProto]: ...

    @property
    def nested_type(self) -> google___protobuf___internal___containers___RepeatedCompositeFieldContainer[type___DescriptorProto]: ...

    @property
    def enum_type(self) -> google___protobuf___internal___containers___RepeatedCompositeFieldContainer[type___EnumDescriptorProto]: ...

    @property
    def extension_range(self) -> google___protobuf___internal___containers___RepeatedCompositeFieldContainer[type___DescriptorProto.ExtensionRange]: ...

    @property
    def oneof_decl(self) -> google___protobuf___internal___containers___RepeatedCompositeFieldContainer[type___OneofDescriptorProto]: ...

    @property
    def options(self) -> type___MessageOptions: ...

    @property
    def reserved_range(self) -> google___protobuf___internal___containers___RepeatedCompositeFieldContainer[type___DescriptorProto.ReservedRange]: ...

    def __init__(self,
        *,
        name : typing___Optional[typing___Text] = None,
        field : typing___Optional[typing___Iterable[type___FieldDescriptorProto]] = None,
        extension : typing___Optional[typing___Iterable[type___FieldDescriptorProto]] = None,
        nested_type : typing___Optional[typing___Iterable[type___DescriptorProto]] = None,
        enum_type : typing___Optional[typing___Iterable[type___EnumDescriptorProto]] = None,
        extension_range : typing___Optional[typing___Iterable[type___DescriptorProto.ExtensionRange]] = None,
        oneof_decl : typing___Optional[typing___Iterable[type___OneofDescriptorProto]] = None,
        options : typing___Optional[type___MessageOptions] = None,
        reserved_range : typing___Optional[typing___Iterable[type___DescriptorProto.ReservedRange]] = None,
        reserved_name : typing___Optional[typing___Iterable[typing___Text]] = None,
        ) -> None: ...
    def HasField(self, field_name: typing_extensions___Literal[u"name",b"name",u"options",b"options"]) -> builtin___bool: ...
    def ClearField(self, field_name: typing_extensions___Literal[u"enum_type",b"enum_type",u"extension",b"extension",u"extension_range",b"extension_range",u"field",b"field",u"name",b"name",u"nested_type",b"nested_type",u"oneof_decl",b"oneof_decl",u"options",b"options",u"reserved_name",b"reserved_name",u"reserved_range",b"reserved_range"]) -> None: ...
type___DescriptorProto = DescriptorProto

class ExtensionRangeOptions(google___protobuf___message___Message):
    DESCRIPTOR: google___protobuf___descriptor___Descriptor = ...

    @property
    def uninterpreted_option(self) -> google___protobuf___internal___containers___RepeatedCompositeFieldContainer[type___UninterpretedOption]: ...

    def __init__(self,
        *,
        uninterpreted_option : typing___Optional[typing___Iterable[type___UninterpretedOption]] = None,
        ) -> None: ...
    def ClearField(self, field_name: typing_extensions___Literal[u"uninterpreted_option",b"uninterpreted_option"]) -> None: ...
type___ExtensionRangeOptions = ExtensionRangeOptions

class FieldDescriptorProto(google___protobuf___message___Message):
    DESCRIPTOR: google___protobuf___descriptor___Descriptor = ...
    TypeValue = typing___NewType('TypeValue', builtin___int)
    type___TypeValue = TypeValue
    Type: _Type
    class _Type(google___protobuf___internal___enum_type_wrapper____EnumTypeWrapper[FieldDescriptorProto.TypeValue]):
        DESCRIPTOR: google___protobuf___descriptor___EnumDescriptor = ...
        TYPE_DOUBLE = typing___cast(FieldDescriptorProto.TypeValue, 1)
        TYPE_FLOAT = typing___cast(FieldDescriptorProto.TypeValue, 2)
        TYPE_INT64 = typing___cast(FieldDescriptorProto.TypeValue, 3)
        TYPE_UINT64 = typing___cast(FieldDescriptorProto.TypeValue, 4)
        TYPE_INT32 = typing___cast(FieldDescriptorProto.TypeValue, 5)
        TYPE_FIXED64 = typing___cast(FieldDescriptorProto.TypeValue, 6)
        TYPE_FIXED32 = typing___cast(FieldDescriptorProto.TypeValue, 7)
        TYPE_BOOL = typing___cast(FieldDescriptorProto.TypeValue, 8)
        TYPE_STRING = typing___cast(FieldDescriptorProto.TypeValue, 9)
        TYPE_GROUP = typing___cast(FieldDescriptorProto.TypeValue, 10)
        TYPE_MESSAGE = typing___cast(FieldDescriptorProto.TypeValue, 11)
        TYPE_BYTES = typing___cast(FieldDescriptorProto.TypeValue, 12)
        TYPE_UINT32 = typing___cast(FieldDescriptorProto.TypeValue, 13)
        TYPE_ENUM = typing___cast(FieldDescriptorProto.TypeValue, 14)
        TYPE_SFIXED32 = typing___cast(FieldDescriptorProto.TypeValue, 15)
        TYPE_SFIXED64 = typing___cast(FieldDescriptorProto.TypeValue, 16)
        TYPE_SINT32 = typing___cast(FieldDescriptorProto.TypeValue, 17)
        TYPE_SINT64 = typing___cast(FieldDescriptorProto.TypeValue, 18)
    TYPE_DOUBLE = typing___cast(FieldDescriptorProto.TypeValue, 1)
    TYPE_FLOAT = typing___cast(FieldDescriptorProto.TypeValue, 2)
    TYPE_INT64 = typing___cast(FieldDescriptorProto.TypeValue, 3)
    TYPE_UINT64 = typing___cast(FieldDescriptorProto.TypeValue, 4)
    TYPE_INT32 = typing___cast(FieldDescriptorProto.TypeValue, 5)
    TYPE_FIXED64 = typing___cast(FieldDescriptorProto.TypeValue, 6)
    TYPE_FIXED32 = typing___cast(FieldDescriptorProto.TypeValue, 7)
    TYPE_BOOL = typing___cast(FieldDescriptorProto.TypeValue, 8)
    TYPE_STRING = typing___cast(FieldDescriptorProto.TypeValue, 9)
    TYPE_GROUP = typing___cast(FieldDescriptorProto.TypeValue, 10)
    TYPE_MESSAGE = typing___cast(FieldDescriptorProto.TypeValue, 11)
    TYPE_BYTES = typing___cast(FieldDescriptorProto.TypeValue, 12)
    TYPE_UINT32 = typing___cast(FieldDescriptorProto.TypeValue, 13)
    TYPE_ENUM = typing___cast(FieldDescriptorProto.TypeValue, 14)
    TYPE_SFIXED32 = typing___cast(FieldDescriptorProto.TypeValue, 15)
    TYPE_SFIXED64 = typing___cast(FieldDescriptorProto.TypeValue, 16)
    TYPE_SINT32 = typing___cast(FieldDescriptorProto.TypeValue, 17)
    TYPE_SINT64 = typing___cast(FieldDescriptorProto.TypeValue, 18)

    LabelValue = typing___NewType('LabelValue', builtin___int)
    type___LabelValue = LabelValue
    Label: _Label
    class _Label(google___protobuf___internal___enum_type_wrapper____EnumTypeWrapper[FieldDescriptorProto.LabelValue]):
        DESCRIPTOR: google___protobuf___descriptor___EnumDescriptor = ...
        LABEL_OPTIONAL = typing___cast(FieldDescriptorProto.LabelValue, 1)
        LABEL_REQUIRED = typing___cast(FieldDescriptorProto.LabelValue, 2)
        LABEL_REPEATED = typing___cast(FieldDescriptorProto.LabelValue, 3)
    LABEL_OPTIONAL = typing___cast(FieldDescriptorProto.LabelValue, 1)
    LABEL_REQUIRED = typing___cast(FieldDescriptorProto.LabelValue, 2)
    LABEL_REPEATED = typing___cast(FieldDescriptorProto.LabelValue, 3)

    name: typing___Text = ...
    number: builtin___int = ...
    label: type___FieldDescriptorProto.LabelValue = ...
    type: type___FieldDescriptorProto.TypeValue = ...
    type_name: typing___Text = ...
    extendee: typing___Text = ...
    default_value: typing___Text = ...
    oneof_index: builtin___int = ...
    json_name: typing___Text = ...
    proto3_optional: builtin___bool = ...

    @property
    def options(self) -> type___FieldOptions: ...

    def __init__(self,
        *,
        name : typing___Optional[typing___Text] = None,
        number : typing___Optional[builtin___int] = None,
        label : typing___Optional[type___FieldDescriptorProto.LabelValue] = None,
        type : typing___Optional[type___FieldDescriptorProto.TypeValue] = None,
        type_name : typing___Optional[typing___Text] = None,
        extendee : typing___Optional[typing___Text] = None,
        default_value : typing___Optional[typing___Text] = None,
        oneof_index : typing___Optional[builtin___int] = None,
        json_name : typing___Optional[typing___Text] = None,
        options : typing___Optional[type___FieldOptions] = None,
        proto3_optional : typing___Optional[builtin___bool] = None,
        ) -> None: ...
    def HasField(self, field_name: typing_extensions___Literal[u"default_value",b"default_value",u"extendee",b"extendee",u"json_name",b"json_name",u"label",b"label",u"name",b"name",u"number",b"number",u"oneof_index",b"oneof_index",u"options",b"options",u"proto3_optional",b"proto3_optional",u"type",b"type",u"type_name",b"type_name"]) -> builtin___bool: ...
    def ClearField(self, field_name: typing_extensions___Literal[u"default_value",b"default_value",u"extendee",b"extendee",u"json_name",b"json_name",u"label",b"label",u"name",b"name",u"number",b"number",u"oneof_index",b"oneof_index",u"options",b"options",u"proto3_optional",b"proto3_optional",u"type",b"type",u"type_name",b"type_name"]) -> None: ...
type___FieldDescriptorProto = FieldDescriptorProto

class OneofDescriptorProto(google___protobuf___message___Message):
    DESCRIPTOR: google___protobuf___descriptor___Descriptor = ...
    name: typing___Text = ...

    @property
    def options(self) -> type___OneofOptions: ...

    def __init__(self,
        *,
        name : typing___Optional[typing___Text] = None,
        options : typing___Optional[type___OneofOptions] = None,
        ) -> None: ...
    def HasField(self, field_name: typing_extensions___Literal[u"name",b"name",u"options",b"options"]) -> builtin___bool: ...
    def ClearField(self, field_name: typing_extensions___Literal[u"name",b"name",u"options",b"options"]) -> None: ...
type___OneofDescriptorProto = OneofDescriptorProto

class EnumDescriptorProto(google___protobuf___message___Message):
    DESCRIPTOR: google___protobuf___descriptor___Descriptor = ...
    class EnumReservedRange(google___protobuf___message___Message):
        DESCRIPTOR: google___protobuf___descriptor___Descriptor = ...
        start: builtin___int = ...
        end: builtin___int = ...

        def __init__(self,
            *,
            start : typing___Optional[builtin___int] = None,
            end : typing___Optional[builtin___int] = None,
            ) -> None: ...
        def HasField(self, field_name: typing_extensions___Literal[u"end",b"end",u"start",b"start"]) -> builtin___bool: ...
        def ClearField(self, field_name: typing_extensions___Literal[u"end",b"end",u"start",b"start"]) -> None: ...
    type___EnumReservedRange = EnumReservedRange

    name: typing___Text = ...
    reserved_name: google___protobuf___internal___containers___RepeatedScalarFieldContainer[typing___Text] = ...

    @property
    def value(self) -> google___protobuf___internal___containers___RepeatedCompositeFieldContainer[type___EnumValueDescriptorProto]: ...

    @property
    def options(self) -> type___EnumOptions: ...

    @property
    def reserved_range(self) -> google___protobuf___internal___containers___RepeatedCompositeFieldContainer[type___EnumDescriptorProto.EnumReservedRange]: ...

    def __init__(self,
        *,
        name : typing___Optional[typing___Text] = None,
        value : typing___Optional[typing___Iterable[type___EnumValueDescriptorProto]] = None,
        options : typing___Optional[type___EnumOptions] = None,
        reserved_range : typing___Optional[typing___Iterable[type___EnumDescriptorProto.EnumReservedRange]] = None,
        reserved_name : typing___Optional[typing___Iterable[typing___Text]] = None,
        ) -> None: ...
    def HasField(self, field_name: typing_extensions___Literal[u"name",b"name",u"options",b"options"]) -> builtin___bool: ...
    def ClearField(self, field_name: typing_extensions___Literal[u"name",b"name",u"options",b"options",u"reserved_name",b"reserved_name",u"reserved_range",b"reserved_range",u"value",b"value"]) -> None: ...
type___EnumDescriptorProto = EnumDescriptorProto

class EnumValueDescriptorProto(google___protobuf___message___Message):
    DESCRIPTOR: google___protobuf___descriptor___Descriptor = ...
    name: typing___Text = ...
    number: builtin___int = ...

    @property
    def options(self) -> type___EnumValueOptions: ...

    def __init__(self,
        *,
        name : typing___Optional[typing___Text] = None,
        number : typing___Optional[builtin___int] = None,
        options : typing___Optional[type___EnumValueOptions] = None,
        ) -> None: ...
    def HasField(self, field_name: typing_extensions___Literal[u"name",b"name",u"number",b"number",u"options",b"options"]) -> builtin___bool: ...
    def ClearField(self, field_name: typing_extensions___Literal[u"name",b"name",u"number",b"number",u"options",b"options"]) -> None: ...
type___EnumValueDescriptorProto = EnumValueDescriptorProto

class ServiceDescriptorProto(google___protobuf___message___Message):
    DESCRIPTOR: google___protobuf___descriptor___Descriptor = ...
    name: typing___Text = ...

    @property
    def method(self) -> google___protobuf___internal___containers___RepeatedCompositeFieldContainer[type___MethodDescriptorProto]: ...

    @property
    def options(self) -> type___ServiceOptions: ...

    def __init__(self,
        *,
        name : typing___Optional[typing___Text] = None,
        method : typing___Optional[typing___Iterable[type___MethodDescriptorProto]] = None,
        options : typing___Optional[type___ServiceOptions] = None,
        ) -> None: ...
    def HasField(self, field_name: typing_extensions___Literal[u"name",b"name",u"options",b"options"]) -> builtin___bool: ...
    def ClearField(self, field_name: typing_extensions___Literal[u"method",b"method",u"name",b"name",u"options",b"options"]) -> None: ...
type___ServiceDescriptorProto = ServiceDescriptorProto

class MethodDescriptorProto(google___protobuf___message___Message):
    DESCRIPTOR: google___protobuf___descriptor___Descriptor = ...
    name: typing___Text = ...
    input_type: typing___Text = ...
    output_type: typing___Text = ...
    client_streaming: builtin___bool = ...
    server_streaming: builtin___bool = ...

    @property
    def options(self) -> type___MethodOptions: ...

    def __init__(self,
        *,
        name : typing___Optional[typing___Text] = None,
        input_type : typing___Optional[typing___Text] = None,
        output_type : typing___Optional[typing___Text] = None,
        options : typing___Optional[type___MethodOptions] = None,
        client_streaming : typing___Optional[builtin___bool] = None,
        server_streaming : typing___Optional[builtin___bool] = None,
        ) -> None: ...
    def HasField(self, field_name: typing_extensions___Literal[u"client_streaming",b"client_streaming",u"input_type",b"input_type",u"name",b"name",u"options",b"options",u"output_type",b"output_type",u"server_streaming",b"server_streaming"]) -> builtin___bool: ...
    def ClearField(self, field_name: typing_extensions___Literal[u"client_streaming",b"client_streaming",u"input_type",b"input_type",u"name",b"name",u"options",b"options",u"output_type",b"output_type",u"server_streaming",b"server_streaming"]) -> None: ...
type___MethodDescriptorProto = MethodDescriptorProto

class FileOptions(google___protobuf___message___Message):
    DESCRIPTOR: google___protobuf___descriptor___Descriptor = ...
    OptimizeModeValue = typing___NewType('OptimizeModeValue', builtin___int)
    type___OptimizeModeValue = OptimizeModeValue
    OptimizeMode: _OptimizeMode
    class _OptimizeMode(google___protobuf___internal___enum_type_wrapper____EnumTypeWrapper[FileOptions.OptimizeModeValue]):
        DESCRIPTOR: google___protobuf___descriptor___EnumDescriptor = ...
        SPEED = typing___cast(FileOptions.OptimizeModeValue, 1)
        CODE_SIZE = typing___cast(FileOptions.OptimizeModeValue, 2)
        LITE_RUNTIME = typing___cast(FileOptions.OptimizeModeValue, 3)
    SPEED = typing___cast(FileOptions.OptimizeModeValue, 1)
    CODE_SIZE = typing___cast(FileOptions.OptimizeModeValue, 2)
    LITE_RUNTIME = typing___cast(FileOptions.OptimizeModeValue, 3)

    java_package: typing___Text = ...
    java_outer_classname: typing___Text = ...
    java_multiple_files: builtin___bool = ...
    java_generate_equals_and_hash: builtin___bool = ...
    java_string_check_utf8: builtin___bool = ...
    optimize_for: type___FileOptions.OptimizeModeValue = ...
    go_package: typing___Text = ...
    cc_generic_services: builtin___bool = ...
    java_generic_services: builtin___bool = ...
    py_generic_services: builtin___bool = ...
    php_generic_services: builtin___bool = ...
    deprecated: builtin___bool = ...
    cc_enable_arenas: builtin___bool = ...
    objc_class_prefix: typing___Text = ...
    csharp_namespace: typing___Text = ...
    swift_prefix: typing___Text = ...
    php_class_prefix: typing___Text = ...
    php_namespace: typing___Text = ...
    php_metadata_namespace: typing___Text = ...
    ruby_package: typing___Text = ...

    @property
    def uninterpreted_option(self) -> google___protobuf___internal___containers___RepeatedCompositeFieldContainer[type___UninterpretedOption]: ...

    def __init__(self,
        *,
        java_package : typing___Optional[typing___Text] = None,
        java_outer_classname : typing___Optional[typing___Text] = None,
        java_multiple_files : typing___Optional[builtin___bool] = None,
        java_generate_equals_and_hash : typing___Optional[builtin___bool] = None,
        java_string_check_utf8 : typing___Optional[builtin___bool] = None,
        optimize_for : typing___Optional[type___FileOptions.OptimizeModeValue] = None,
        go_package : typing___Optional[typing___Text] = None,
        cc_generic_services : typing___Optional[builtin___bool] = None,
        java_generic_services : typing___Optional[builtin___bool] = None,
        py_generic_services : typing___Optional[builtin___bool] = None,
        php_generic_services : typing___Optional[builtin___bool] = None,
        deprecated : typing___Optional[builtin___bool] = None,
        cc_enable_arenas : typing___Optional[builtin___bool] = None,
        objc_class_prefix : typing___Optional[typing___Text] = None,
        csharp_namespace : typing___Optional[typing___Text] = None,
        swift_prefix : typing___Optional[typing___Text] = None,
        php_class_prefix : typing___Optional[typing___Text] = None,
        php_namespace : typing___Optional[typing___Text] = None,
        php_metadata_namespace : typing___Optional[typing___Text] = None,
        ruby_package : typing___Optional[typing___Text] = None,
        uninterpreted_option : typing___Optional[typing___Iterable[type___UninterpretedOption]] = None,
        ) -> None: ...
    def HasField(self, field_name: typing_extensions___Literal[u"cc_enable_arenas",b"cc_enable_arenas",u"cc_generic_services",b"cc_generic_services",u"csharp_namespace",b"csharp_namespace",u"deprecated",b"deprecated",u"go_package",b"go_package",u"java_generate_equals_and_hash",b"java_generate_equals_and_hash",u"java_generic_services",b"java_generic_services",u"java_multiple_files",b"java_multiple_files",u"java_outer_classname",b"java_outer_classname",u"java_package",b"java_package",u"java_string_check_utf8",b"java_string_check_utf8",u"objc_class_prefix",b"objc_class_prefix",u"optimize_for",b"optimize_for",u"php_class_prefix",b"php_class_prefix",u"php_generic_services",b"php_generic_services",u"php_metadata_namespace",b"php_metadata_namespace",u"php_namespace",b"php_namespace",u"py_generic_services",b"py_generic_services",u"ruby_package",b"ruby_package",u"swift_prefix",b"swift_prefix"]) -> builtin___bool: ...
    def ClearField(self, field_name: typing_extensions___Literal[u"cc_enable_arenas",b"cc_enable_arenas",u"cc_generic_services",b"cc_generic_services",u"csharp_namespace",b"csharp_namespace",u"deprecated",b"deprecated",u"go_package",b"go_package",u"java_generate_equals_and_hash",b"java_generate_equals_and_hash",u"java_generic_services",b"java_generic_services",u"java_multiple_files",b"java_multiple_files",u"java_outer_classname",b"java_outer_classname",u"java_package",b"java_package",u"java_string_check_utf8",b"java_string_check_utf8",u"objc_class_prefix",b"objc_class_prefix",u"optimize_for",b"optimize_for",u"php_class_prefix",b"php_class_prefix",u"php_generic_services",b"php_generic_services",u"php_metadata_namespace",b"php_metadata_namespace",u"php_namespace",b"php_namespace",u"py_generic_services",b"py_generic_services",u"ruby_package",b"ruby_package",u"swift_prefix",b"swift_prefix",u"uninterpreted_option",b"uninterpreted_option"]) -> None: ...
type___FileOptions = FileOptions

class MessageOptions(google___protobuf___message___Message):
    DESCRIPTOR: google___protobuf___descriptor___Descriptor = ...
    message_set_wire_format: builtin___bool = ...
    no_standard_descriptor_accessor: builtin___bool = ...
    deprecated: builtin___bool = ...
    map_entry: builtin___bool = ...

    @property
    def uninterpreted_option(self) -> google___protobuf___internal___containers___RepeatedCompositeFieldContainer[type___UninterpretedOption]: ...

    def __init__(self,
        *,
        message_set_wire_format : typing___Optional[builtin___bool] = None,
        no_standard_descriptor_accessor : typing___Optional[builtin___bool] = None,
        deprecated : typing___Optional[builtin___bool] = None,
        map_entry : typing___Optional[builtin___bool] = None,
        uninterpreted_option : typing___Optional[typing___Iterable[type___UninterpretedOption]] = None,
        ) -> None: ...
    def HasField(self, field_name: typing_extensions___Literal[u"deprecated",b"deprecated",u"map_entry",b"map_entry",u"message_set_wire_format",b"message_set_wire_format",u"no_standard_descriptor_accessor",b"no_standard_descriptor_accessor"]) -> builtin___bool: ...
    def ClearField(self, field_name: typing_extensions___Literal[u"deprecated",b"deprecated",u"map_entry",b"map_entry",u"message_set_wire_format",b"message_set_wire_format",u"no_standard_descriptor_accessor",b"no_standard_descriptor_accessor",u"uninterpreted_option",b"uninterpreted_option"]) -> None: ...
type___MessageOptions = MessageOptions

class FieldOptions(google___protobuf___message___Message):
    DESCRIPTOR: google___protobuf___descriptor___Descriptor = ...
    CTypeValue = typing___NewType('CTypeValue', builtin___int)
    type___CTypeValue = CTypeValue
    CType: _CType
    class _CType(google___protobuf___internal___enum_type_wrapper____EnumTypeWrapper[FieldOptions.CTypeValue]):
        DESCRIPTOR: google___protobuf___descriptor___EnumDescriptor = ...
        STRING = typing___cast(FieldOptions.CTypeValue, 0)
        CORD = typing___cast(FieldOptions.CTypeValue, 1)
        STRING_PIECE = typing___cast(FieldOptions.CTypeValue, 2)
    STRING = typing___cast(FieldOptions.CTypeValue, 0)
    CORD = typing___cast(FieldOptions.CTypeValue, 1)
    STRING_PIECE = typing___cast(FieldOptions.CTypeValue, 2)

    JSTypeValue = typing___NewType('JSTypeValue', builtin___int)
    type___JSTypeValue = JSTypeValue
    JSType: _JSType
    class _JSType(google___protobuf___internal___enum_type_wrapper____EnumTypeWrapper[FieldOptions.JSTypeValue]):
        DESCRIPTOR: google___protobuf___descriptor___EnumDescriptor = ...
        JS_NORMAL = typing___cast(FieldOptions.JSTypeValue, 0)
        JS_STRING = typing___cast(FieldOptions.JSTypeValue, 1)
        JS_NUMBER = typing___cast(FieldOptions.JSTypeValue, 2)
    JS_NORMAL = typing___cast(FieldOptions.JSTypeValue, 0)
    JS_STRING = typing___cast(FieldOptions.JSTypeValue, 1)
    JS_NUMBER = typing___cast(FieldOptions.JSTypeValue, 2)

    ctype: type___FieldOptions.CTypeValue = ...
    packed: builtin___bool = ...
    jstype: type___FieldOptions.JSTypeValue = ...
    lazy: builtin___bool = ...
    deprecated: builtin___bool = ...
    weak: builtin___bool = ...

    @property
    def uninterpreted_option(self) -> google___protobuf___internal___containers___RepeatedCompositeFieldContainer[type___UninterpretedOption]: ...

    def __init__(self,
        *,
        ctype : typing___Optional[type___FieldOptions.CTypeValue] = None,
        packed : typing___Optional[builtin___bool] = None,
        jstype : typing___Optional[type___FieldOptions.JSTypeValue] = None,
        lazy : typing___Optional[builtin___bool] = None,
        deprecated : typing___Optional[builtin___bool] = None,
        weak : typing___Optional[builtin___bool] = None,
        uninterpreted_option : typing___Optional[typing___Iterable[type___UninterpretedOption]] = None,
        ) -> None: ...
    def HasField(self, field_name: typing_extensions___Literal[u"ctype",b"ctype",u"deprecated",b"deprecated",u"jstype",b"jstype",u"lazy",b"lazy",u"packed",b"packed",u"weak",b"weak"]) -> builtin___bool: ...
    def ClearField(self, field_name: typing_extensions___Literal[u"ctype",b"ctype",u"deprecated",b"deprecated",u"jstype",b"jstype",u"lazy",b"lazy",u"packed",b"packed",u"uninterpreted_option",b"uninterpreted_option",u"weak",b"weak"]) -> None: ...
type___FieldOptions = FieldOptions

class OneofOptions(google___protobuf___message___Message):
    DESCRIPTOR: google___protobuf___descriptor___Descriptor = ...

    @property
    def uninterpreted_option(self) -> google___protobuf___internal___containers___RepeatedCompositeFieldContainer[type___UninterpretedOption]: ...

    def __init__(self,
        *,
        uninterpreted_option : typing___Optional[typing___Iterable[type___UninterpretedOption]] = None,
        ) -> None: ...
    def ClearField(self, field_name: typing_extensions___Literal[u"uninterpreted_option",b"uninterpreted_option"]) -> None: ...
type___OneofOptions = OneofOptions

class EnumOptions(google___protobuf___message___Message):
    DESCRIPTOR: google___protobuf___descriptor___Descriptor = ...
    allow_alias: builtin___bool = ...
    deprecated: builtin___bool = ...

    @property
    def uninterpreted_option(self) -> google___protobuf___internal___containers___RepeatedCompositeFieldContainer[type___UninterpretedOption]: ...

    def __init__(self,
        *,
        allow_alias : typing___Optional[builtin___bool] = None,
        deprecated : typing___Optional[builtin___bool] = None,
        uninterpreted_option : typing___Optional[typing___Iterable[type___UninterpretedOption]] = None,
        ) -> None: ...
    def HasField(self, field_name: typing_extensions___Literal[u"allow_alias",b"allow_alias",u"deprecated",b"deprecated"]) -> builtin___bool: ...
    def ClearField(self, field_name: typing_extensions___Literal[u"allow_alias",b"allow_alias",u"deprecated",b"deprecated",u"uninterpreted_option",b"uninterpreted_option"]) -> None: ...
type___EnumOptions = EnumOptions

class EnumValueOptions(google___protobuf___message___Message):
    DESCRIPTOR: google___protobuf___descriptor___Descriptor = ...
    deprecated: builtin___bool = ...

    @property
    def uninterpreted_option(self) -> google___protobuf___internal___containers___RepeatedCompositeFieldContainer[type___UninterpretedOption]: ...

    def __init__(self,
        *,
        deprecated : typing___Optional[builtin___bool] = None,
        uninterpreted_option : typing___Optional[typing___Iterable[type___UninterpretedOption]] = None,
        ) -> None: ...
    def HasField(self, field_name: typing_extensions___Literal[u"deprecated",b"deprecated"]) -> builtin___bool: ...
    def ClearField(self, field_name: typing_extensions___Literal[u"deprecated",b"deprecated",u"uninterpreted_option",b"uninterpreted_option"]) -> None: ...
type___EnumValueOptions = EnumValueOptions

class ServiceOptions(google___protobuf___message___Message):
    DESCRIPTOR: google___protobuf___descriptor___Descriptor = ...
    deprecated: builtin___bool = ...

    @property
    def uninterpreted_option(self) -> google___protobuf___internal___containers___RepeatedCompositeFieldContainer[type___UninterpretedOption]: ...

    def __init__(self,
        *,
        deprecated : typing___Optional[builtin___bool] = None,
        uninterpreted_option : typing___Optional[typing___Iterable[type___UninterpretedOption]] = None,
        ) -> None: ...
    def HasField(self, field_name: typing_extensions___Literal[u"deprecated",b"deprecated"]) -> builtin___bool: ...
    def ClearField(self, field_name: typing_extensions___Literal[u"deprecated",b"deprecated",u"uninterpreted_option",b"uninterpreted_option"]) -> None: ...
type___ServiceOptions = ServiceOptions

class MethodOptions(google___protobuf___message___Message):
    DESCRIPTOR: google___protobuf___descriptor___Descriptor = ...
    IdempotencyLevelValue = typing___NewType('IdempotencyLevelValue', builtin___int)
    type___IdempotencyLevelValue = IdempotencyLevelValue
    IdempotencyLevel: _IdempotencyLevel
    class _IdempotencyLevel(google___protobuf___internal___enum_type_wrapper____EnumTypeWrapper[MethodOptions.IdempotencyLevelValue]):
        DESCRIPTOR: google___protobuf___descriptor___EnumDescriptor = ...
        IDEMPOTENCY_UNKNOWN = typing___cast(MethodOptions.IdempotencyLevelValue, 0)
        NO_SIDE_EFFECTS = typing___cast(MethodOptions.IdempotencyLevelValue, 1)
        IDEMPOTENT = typing___cast(MethodOptions.IdempotencyLevelValue, 2)
    IDEMPOTENCY_UNKNOWN = typing___cast(MethodOptions.IdempotencyLevelValue, 0)
    NO_SIDE_EFFECTS = typing___cast(MethodOptions.IdempotencyLevelValue, 1)
    IDEMPOTENT = typing___cast(MethodOptions.IdempotencyLevelValue, 2)

    deprecated: builtin___bool = ...
    idempotency_level: type___MethodOptions.IdempotencyLevelValue = ...

    @property
    def uninterpreted_option(self) -> google___protobuf___internal___containers___RepeatedCompositeFieldContainer[type___UninterpretedOption]: ...

    def __init__(self,
        *,
        deprecated : typing___Optional[builtin___bool] = None,
        idempotency_level : typing___Optional[type___MethodOptions.IdempotencyLevelValue] = None,
        uninterpreted_option : typing___Optional[typing___Iterable[type___UninterpretedOption]] = None,
        ) -> None: ...
    def HasField(self, field_name: typing_extensions___Literal[u"deprecated",b"deprecated",u"idempotency_level",b"idempotency_level"]) -> builtin___bool: ...
    def ClearField(self, field_name: typing_extensions___Literal[u"deprecated",b"deprecated",u"idempotency_level",b"idempotency_level",u"uninterpreted_option",b"uninterpreted_option"]) -> None: ...
type___MethodOptions = MethodOptions

class UninterpretedOption(google___protobuf___message___Message):
    DESCRIPTOR: google___protobuf___descriptor___Descriptor = ...
    class NamePart(google___protobuf___message___Message):
        DESCRIPTOR: google___protobuf___descriptor___Descriptor = ...
        name_part: typing___Text = ...
        is_extension: builtin___bool = ...

        def __init__(self,
            *,
            name_part : typing___Optional[typing___Text] = None,
            is_extension : typing___Optional[builtin___bool] = None,
            ) -> None: ...
        def HasField(self, field_name: typing_extensions___Literal[u"is_extension",b"is_extension",u"name_part",b"name_part"]) -> builtin___bool: ...
        def ClearField(self, field_name: typing_extensions___Literal[u"is_extension",b"is_extension",u"name_part",b"name_part"]) -> None: ...
    type___NamePart = NamePart

    identifier_value: typing___Text = ...
    positive_int_value: builtin___int = ...
    negative_int_value: builtin___int = ...
    double_value: builtin___float = ...
    string_value: builtin___bytes = ...
    aggregate_value: typing___Text = ...

    @property
    def name(self) -> google___protobuf___internal___containers___RepeatedCompositeFieldContainer[type___UninterpretedOption.NamePart]: ...

    def __init__(self,
        *,
        name : typing___Optional[typing___Iterable[type___UninterpretedOption.NamePart]] = None,
        identifier_value : typing___Optional[typing___Text] = None,
        positive_int_value : typing___Optional[builtin___int] = None,
        negative_int_value : typing___Optional[builtin___int] = None,
        double_value : typing___Optional[builtin___float] = None,
        string_value : typing___Optional[builtin___bytes] = None,
        aggregate_value : typing___Optional[typing___Text] = None,
        ) -> None: ...
    def HasField(self, field_name: typing_extensions___Literal[u"aggregate_value",b"aggregate_value",u"double_value",b"double_value",u"identifier_value",b"identifier_value",u"negative_int_value",b"negative_int_value",u"positive_int_value",b"positive_int_value",u"string_value",b"string_value"]) -> builtin___bool: ...
    def ClearField(self, field_name: typing_extensions___Literal[u"aggregate_value",b"aggregate_value",u"double_value",b"double_value",u"identifier_value",b"identifier_value",u"name",b"name",u"negative_int_value",b"negative_int_value",u"positive_int_value",b"positive_int_value",u"string_value",b"string_value"]) -> None: ...
type___UninterpretedOption = UninterpretedOption

class SourceCodeInfo(google___protobuf___message___Message):
    DESCRIPTOR: google___protobuf___descriptor___Descriptor = ...
    class Location(google___protobuf___message___Message):
        DESCRIPTOR: google___protobuf___descriptor___Descriptor = ...
        path: google___protobuf___internal___containers___RepeatedScalarFieldContainer[builtin___int] = ...
        span: google___protobuf___internal___containers___RepeatedScalarFieldContainer[builtin___int] = ...
        leading_comments: typing___Text = ...
        trailing_comments: typing___Text = ...
        leading_detached_comments: google___protobuf___internal___containers___RepeatedScalarFieldContainer[typing___Text] = ...

        def __init__(self,
            *,
            path : typing___Optional[typing___Iterable[builtin___int]] = None,
            span : typing___Optional[typing___Iterable[builtin___int]] = None,
            leading_comments : typing___Optional[typing___Text] = None,
            trailing_comments : typing___Optional[typing___Text] = None,
            leading_detached_comments : typing___Optional[typing___Iterable[typing___Text]] = None,
            ) -> None: ...
        def HasField(self, field_name: typing_extensions___Literal[u"leading_comments",b"leading_comments",u"trailing_comments",b"trailing_comments"]) -> builtin___bool: ...
        def ClearField(self, field_name: typing_extensions___Literal[u"leading_comments",b"leading_comments",u"leading_detached_comments",b"leading_detached_comments",u"path",b"path",u"span",b"span",u"trailing_comments",b"trailing_comments"]) -> None: ...
    type___Location = Location


    @property
    def location(self) -> google___protobuf___internal___containers___RepeatedCompositeFieldContainer[type___SourceCodeInfo.Location]: ...

    def __init__(self,
        *,
        location : typing___Optional[typing___Iterable[type___SourceCodeInfo.Location]] = None,
        ) -> None: ...
    def ClearField(self, field_name: typing_extensions___Literal[u"location",b"location"]) -> None: ...
type___SourceCodeInfo = SourceCodeInfo

class GeneratedCodeInfo(google___protobuf___message___Message):
    DESCRIPTOR: google___protobuf___descriptor___Descriptor = ...
    class Annotation(google___protobuf___message___Message):
        DESCRIPTOR: google___protobuf___descriptor___Descriptor = ...
        path: google___protobuf___internal___containers___RepeatedScalarFieldContainer[builtin___int] = ...
        source_file: typing___Text = ...
        begin: builtin___int = ...
        end: builtin___int = ...

        def __init__(self,
            *,
            path : typing___Optional[typing___Iterable[builtin___int]] = None,
            source_file : typing___Optional[typing___Text] = None,
            begin : typing___Optional[builtin___int] = None,
            end : typing___Optional[builtin___int] = None,
            ) -> None: ...
        def HasField(self, field_name: typing_extensions___Literal[u"begin",b"begin",u"end",b"end",u"source_file",b"source_file"]) -> builtin___bool: ...
        def ClearField(self, field_name: typing_extensions___Literal[u"begin",b"begin",u"end",b"end",u"path",b"path",u"source_file",b"source_file"]) -> None: ...
    type___Annotation = Annotation


    @property
    def annotation(self) -> google___protobuf___internal___containers___RepeatedCompositeFieldContainer[type___GeneratedCodeInfo.Annotation]: ...

    def __init__(self,
        *,
        annotation : typing___Optional[typing___Iterable[type___GeneratedCodeInfo.Annotation]] = None,
        ) -> None: ...
    def ClearField(self, field_name: typing_extensions___Literal[u"annotation",b"annotation"]) -> None: ...
type___GeneratedCodeInfo = GeneratedCodeInfo



from types import ModuleType
from typing import IO, Any, Callable, Dict, Iterable, List, Optional, Sequence, Text, Union

class Error(Exception): ...

FlagsError = Error

class DuplicateFlag(FlagsError): ...
class CantOpenFlagFileError(FlagsError): ...
class DuplicateFlagCannotPropagateNoneToSwig(DuplicateFlag): ...

class DuplicateFlagError(DuplicateFlag):
    def __init__(self, flagname: str, flag_values: FlagValues, other_flag_values: FlagValues = ...) -> None: ...

class IllegalFlagValueError(FlagsError): ...

IllegalFlagValue = IllegalFlagValueError

class UnrecognizedFlag(FlagsError): ...

class UnrecognizedFlagError(UnrecognizedFlag):
    def __init__(self, flagname: str, flagvalue: str = ...) -> None: ...

def get_help_width() -> int: ...

GetHelpWidth = get_help_width

def text_wrap(text: str, length: int = ..., indent: str = ..., firstline_indent: str = ..., tabs: str = ...) -> str: ...

TextWrap = text_wrap

def doc_to_help(doc: str) -> str: ...

DocToHelp = doc_to_help

class FlagValues:
    def __init__(self) -> None: ...
    def UseGnuGetOpt(self, use_gnu_getopt: bool = ...) -> None: ...
    def is_gnu_getopt(self) -> bool: ...
    IsGnuGetOpt = is_gnu_getopt
    # TODO dict type
    def FlagDict(self) -> Dict[Any, Any]: ...
    def flags_by_module_dict(self) -> Dict[str, List[Flag]]: ...
    FlagsByModuleDict = flags_by_module_dict
    def flags_by_module_id_dict(self) -> Dict[int, List[Flag]]: ...
    FlagsByModuleIdDict = flags_by_module_id_dict
    def key_flags_by_module_dict(self) -> Dict[str, List[Flag]]: ...
    KeyFlagsByModuleDict = key_flags_by_module_dict
    def find_module_defining_flag(self, flagname: str, default: str = ...) -> str: ...
    FindModuleDefiningFlag = find_module_defining_flag
    def find_module_id_defining_flag(self, flagname: str, default: int = ...) -> int: ...
    FindModuleIdDefiningFlag = find_module_id_defining_flag
    def append_flag_values(self, flag_values: FlagValues) -> None: ...
    AppendFlagValues = append_flag_values
    def remove_flag_values(self, flag_values: FlagValues) -> None: ...
    RemoveFlagValues = remove_flag_values
    def __setitem__(self, name: str, flag: Flag) -> None: ...
    def __getitem__(self, name: str) -> Flag: ...
    def __getattr__(self, name: str) -> Any: ...
    def __setattr__(self, name: str, value: Any) -> None: ...
    def __delattr__(self, flag_name: str) -> None: ...
    def set_default(self, name: str, value: Any) -> None: ...
    SetDefault = set_default
    def __contains__(self, name: str) -> bool: ...
    has_key = __contains__
    def __iter__(self) -> Iterable[str]: ...
    def __call__(self, argv: List[str], known_only: bool = ...) -> List[str]: ...
    def reset(self) -> None: ...
    Reset = reset
    def RegisteredFlags(self) -> List[str]: ...
    def flag_values_dict(self) -> Dict[str, Any]: ...
    FlagValuesDict = flag_values_dict
    def __str__(self) -> str: ...
    def GetHelp(self, prefix: str = ...) -> str: ...
    def module_help(self, module: Union[ModuleType, str]) -> str: ...
    ModuleHelp = module_help
    def main_module_help(self) -> str: ...
    MainModuleHelp = main_module_help
    def get(self, name: str, default: Any) -> Any: ...
    def ShortestUniquePrefixes(self, fl: Dict[str, Flag]) -> Dict[str, str]: ...
    def ExtractFilename(self, flagfile_str: str) -> str: ...
    def read_flags_from_files(self, argv: List[str], force_gnu: bool = ...) -> List[str]: ...
    ReadFlagsFromFiles = read_flags_from_files
    def flags_into_string(self) -> str: ...
    FlagsIntoString = flags_into_string
    def append_flags_into_file(self, filename: str) -> None: ...
    AppendFlagsIntoFile = append_flags_into_file
    def write_help_in_xml_format(self, outfile: IO[str] = ...) -> None: ...
    WriteHelpInXMLFormat = write_help_in_xml_format
    # TODO validator: gflags_validators.Validator
    def AddValidator(self, validator: Any) -> None: ...
    def is_parsed(self) -> bool: ...
    IsParsed = is_parsed

FLAGS: FlagValues

class Flag:
    name: str
    default: Any
    default_as_str: str
    value: Any
    help: str
    short_name: str
    boolean = False
    present = False
    parser: ArgumentParser
    serializer: ArgumentSerializer
    allow_override = False
    def __init__(
        self,
        parser: ArgumentParser,
        serializer: ArgumentSerializer,
        name: str,
        default: Optional[str],
        help_string: str,
        short_name: str = ...,
        boolean: bool = ...,
        allow_override: bool = ...,
    ) -> None: ...
    def Parse(self, argument: Any) -> Any: ...
    def Unparse(self) -> None: ...
    def Serialize(self) -> str: ...
    def SetDefault(self, value: Any) -> None: ...
    def Type(self) -> str: ...
    def WriteInfoInXMLFormat(self, outfile: IO[str], module_name: str, is_key: bool = ..., indent: str = ...) -> None: ...

class ArgumentParser(object):
    syntactic_help: str
    # TODO what is this
    def parse(self, argument: Any) -> Any: ...
    Parser = parse
    def flag_type(self) -> str: ...
    Type = flag_type
    def WriteCustomInfoInXMLFormat(self, outfile: IO[str], indent: str) -> None: ...

class ArgumentSerializer:
    def Serialize(self, value: Any) -> Text: ...

class ListSerializer(ArgumentSerializer):
    def __init__(self, list_sep: str) -> None: ...
    def Serialize(self, value: List[Any]) -> str: ...

def register_validator(
    flag_name: str, checker: Callable[[Any], bool], message: str = ..., flag_values: FlagValues = ...
) -> None: ...

RegisterValidator = register_validator

def mark_flag_as_required(flag_name: str, flag_values: FlagValues = ...) -> None: ...

MarkFlagAsRequired = mark_flag_as_required

def mark_flags_as_required(flag_names: Iterable[str], flag_values: FlagValues = ...) -> None: ...

MarkFlagsAsRequired = mark_flags_as_required

def mark_flags_as_mutual_exclusive(flag_names: Iterable[str], required: bool = ..., flag_values: FlagValues = ...) -> None: ...

MarkFlagsAsMutualExclusive = mark_flags_as_mutual_exclusive

def DEFINE(
    parser: ArgumentParser,
    name: str,
    default: Any,
    help: str,
    flag_values: FlagValues = ...,
    serializer: ArgumentSerializer = ...,
    **args: Any,
) -> None: ...
def DEFINE_flag(flag: Flag, flag_values: FlagValues = ...) -> None: ...
def declare_key_flag(flag_name: str, flag_values: FlagValues = ...) -> None: ...

DECLARE_key_flag = declare_key_flag

def adopt_module_key_flags(module: ModuleType, flag_values: FlagValues = ...) -> None: ...

ADOPT_module_key_flags = adopt_module_key_flags

def DEFINE_string(name: str, default: Optional[str], help: str, flag_values: FlagValues = ..., **args: Any) -> None: ...

class BooleanParser(ArgumentParser):
    def Convert(self, argument: Any) -> bool: ...
    def Parse(self, argument: Any) -> bool: ...

class BooleanFlag(Flag):
    def __init__(self, name: str, default: Optional[bool], help: str, short_name: str = ..., **args: Any) -> None: ...

def DEFINE_boolean(name: str, default: Optional[bool], help: str, flag_values: FlagValues = ..., **args: Any) -> None: ...

DEFINE_bool = DEFINE_boolean

class HelpFlag(BooleanFlag):
    def __init__(self) -> None: ...
    def Parse(self, arg: Any) -> None: ...

class HelpXMLFlag(BooleanFlag):
    def __init__(self) -> None: ...
    def Parse(self, arg: Any) -> None: ...

class HelpshortFlag(BooleanFlag):
    def __init__(self) -> None: ...
    def Parse(self, arg: Any) -> None: ...

class NumericParser(ArgumentParser):
    def IsOutsideBounds(self, val: float) -> bool: ...
    def Parse(self, argument: Any) -> float: ...
    def WriteCustomInfoInXMLFormat(self, outfile: IO[str], indent: str) -> None: ...
    def Convert(self, argument: Any) -> Any: ...

class FloatParser(NumericParser):
    number_article: str
    number_name: str
    syntactic_help: str
    def __init__(self, lower_bound: float = ..., upper_bound: float = ...) -> None: ...
    def Convert(self, argument: Any) -> float: ...

def DEFINE_float(
    name: str,
    default: Optional[float],
    help: str,
    lower_bound: float = ...,
    upper_bound: float = ...,
    flag_values: FlagValues = ...,
    **args: Any,
) -> None: ...

class IntegerParser(NumericParser):
    number_article: str
    number_name: str
    syntactic_help: str
    def __init__(self, lower_bound: int = ..., upper_bound: int = ...) -> None: ...
    def Convert(self, argument: Any) -> int: ...

def DEFINE_integer(
    name: str,
    default: Optional[int],
    help: str,
    lower_bound: int = ...,
    upper_bound: int = ...,
    flag_values: FlagValues = ...,
    **args: Any,
) -> None: ...

class EnumParser(ArgumentParser):
    def __init__(self, enum_values: List[str]) -> None: ...
    def Parse(self, argument: Any) -> Any: ...

class EnumFlag(Flag):
    def __init__(
        self, name: str, default: Optional[str], help: str, enum_values: List[str], short_name: str, **args: Any
    ) -> None: ...

def DEFINE_enum(
    name: str, default: Optional[str], enum_values: Iterable[str], help: str, flag_values: FlagValues = ..., **args: Any
) -> None: ...

class BaseListParser(ArgumentParser):
    def __init__(self, token: str = ..., name: str = ...) -> None: ...
    def Parse(self, argument: Any) -> List[Any]: ...

class ListParser(BaseListParser):
    def __init__(self) -> None: ...

class WhitespaceSeparatedListParser(BaseListParser):
    def __init__(self) -> None: ...

def DEFINE_list(name: str, default: Optional[List[str]], help: str, flag_values: FlagValues = ..., **args: Any) -> None: ...
def DEFINE_spaceseplist(
    name: str, default: Optional[List[str]], help: str, flag_values: FlagValues = ..., **args: Any
) -> None: ...

class MultiFlag(Flag):
    def __init__(self, *args: Any, **kwargs: Any) -> None: ...
    def Parse(self, arguments: Any) -> None: ...
    def Serialize(self) -> str: ...

def DEFINE_multi_string(
    name: str, default: Optional[Union[str, List[str]]], help: str, flag_values: FlagValues = ..., **args: Any
) -> None: ...

DEFINE_multistring = DEFINE_multi_string

def DEFINE_multi_integer(
    name: str,
    default: Optional[Union[int, List[int]]],
    help: str,
    lower_bound: int = ...,
    upper_bound: int = ...,
    flag_values: FlagValues = ...,
    **args: Any,
) -> None: ...

DEFINE_multi_int = DEFINE_multi_integer

def DEFINE_multi_float(
    name: str,
    default: Optional[Union[float, List[float]]],
    help: str,
    lower_bound: float = ...,
    upper_bound: float = ...,
    flag_values: FlagValues = ...,
    **args: Any,
) -> None: ...
def DEFINE_multi_enum(
    name: str,
    default: Optional[Union[Sequence[str], str]],
    enum_values: Sequence[str],
    help: str,
    flag_values: FlagValues = ...,
    case_sensitive: bool = ...,
    **args: Any,
) -> None: ...



from typing import Any, Dict, Generic, Iterable, Iterator, Mapping, MutableMapping, Optional, Tuple, TypeVar, Union

_VT = TypeVar("_VT")

class CaseInsensitiveDict(MutableMapping[str, _VT], Generic[_VT]):
    def __init__(self, data: Optional[Union[Mapping[str, _VT], Iterable[Tuple[str, _VT]]]] = ..., **kwargs: _VT) -> None: ...
    def lower_items(self) -> Iterator[Tuple[str, _VT]]: ...
    def __setitem__(self, key: str, value: _VT) -> None: ...
    def __getitem__(self, key: str) -> _VT: ...
    def __delitem__(self, key: str) -> None: ...
    def __iter__(self) -> Iterator[str]: ...
    def __len__(self) -> int: ...
    def copy(self) -> CaseInsensitiveDict[_VT]: ...

class LookupDict(Dict[str, _VT]):
    name: Any
    def __init__(self, name: Any = ...) -> None: ...
    def __getitem__(self, key: str) -> Optional[_VT]: ...  # type: ignore
    def __getattr__(self, attr: str) -> _VT: ...
    def __setattr__(self, attr: str, value: _VT) -> None: ...



import types
from importlib.machinery import ModuleSpec
from typing import Any, List

def create_builtin(__spec: ModuleSpec) -> types.ModuleType: ...
def create_dynamic(__spec: ModuleSpec, __file: Any = ...) -> None: ...
def acquire_lock() -> None: ...
def exec_builtin(__mod: types.ModuleType) -> int: ...
def exec_dynamic(__mod: types.ModuleType) -> int: ...
def extension_suffixes() -> List[str]: ...
def get_frozen_object(__name: str) -> types.CodeType: ...
def init_frozen(__name: str) -> types.ModuleType: ...
def is_builtin(__name: str) -> int: ...
def is_frozen(__name: str) -> bool: ...
def is_frozen_package(__name: str) -> bool: ...
def lock_held() -> bool: ...
def release_lock() -> None: ...



from subprocess import Popen
from typing import List, Optional

from paramiko.util import ClosingContextManager

class ProxyCommand(ClosingContextManager):
    cmd: List[str]
    process: Popen
    timeout: Optional[float]
    def __init__(self, command_line: str) -> None: ...
    def send(self, content: bytes) -> int: ...
    def recv(self, size: int) -> bytes: ...
    def close(self) -> None: ...
    @property
    def closed(self) -> bool: ...
    def settimeout(self, timeout: float) -> None: ...



import datetime
from typing import IO, Any, List, Optional, Text, Tuple, Union

from ..relativedelta import relativedelta
from ._common import _tzinfo as _tzinfo, enfold as enfold, tzname_in_python2 as tzname_in_python2, tzrangebase as tzrangebase

_FileObj = Union[str, Text, IO[str], IO[Text]]

ZERO: datetime.timedelta
EPOCH: datetime.datetime
EPOCHORDINAL: int

class tzutc(datetime.tzinfo):
    def utcoffset(self, dt: Optional[datetime.datetime]) -> Optional[datetime.timedelta]: ...
    def dst(self, dt: Optional[datetime.datetime]) -> Optional[datetime.timedelta]: ...
    def tzname(self, dt: Optional[datetime.datetime]) -> str: ...
    def is_ambiguous(self, dt: Optional[datetime.datetime]) -> bool: ...
    def __eq__(self, other): ...
    __hash__: Any
    def __ne__(self, other): ...
    __reduce__: Any

class tzoffset(datetime.tzinfo):
    def __init__(self, name, offset) -> None: ...
    def utcoffset(self, dt: Optional[datetime.datetime]) -> Optional[datetime.timedelta]: ...
    def dst(self, dt: Optional[datetime.datetime]) -> Optional[datetime.timedelta]: ...
    def is_ambiguous(self, dt: Optional[datetime.datetime]) -> bool: ...
    def tzname(self, dt: Optional[datetime.datetime]) -> str: ...
    def __eq__(self, other): ...
    __hash__: Any
    def __ne__(self, other): ...
    __reduce__: Any
    @classmethod
    def instance(cls, name, offset) -> tzoffset: ...

class tzlocal(_tzinfo):
    def __init__(self) -> None: ...
    def utcoffset(self, dt: Optional[datetime.datetime]) -> Optional[datetime.timedelta]: ...
    def dst(self, dt: Optional[datetime.datetime]) -> Optional[datetime.timedelta]: ...
    def tzname(self, dt: Optional[datetime.datetime]) -> str: ...
    def is_ambiguous(self, dt: Optional[datetime.datetime]) -> bool: ...
    def __eq__(self, other): ...
    __hash__: Any
    def __ne__(self, other): ...
    __reduce__: Any

class _ttinfo:
    def __init__(self) -> None: ...
    def __eq__(self, other): ...
    __hash__: Any
    def __ne__(self, other): ...

class tzfile(_tzinfo):
    def __init__(self, fileobj: _FileObj, filename: Optional[Text] = ...) -> None: ...
    def is_ambiguous(self, dt: Optional[datetime.datetime], idx: Optional[int] = ...) -> bool: ...
    def utcoffset(self, dt: Optional[datetime.datetime]) -> Optional[datetime.timedelta]: ...
    def dst(self, dt: Optional[datetime.datetime]) -> Optional[datetime.timedelta]: ...
    def tzname(self, dt: Optional[datetime.datetime]) -> str: ...
    def __eq__(self, other): ...
    __hash__: Any
    def __ne__(self, other): ...
    def __reduce__(self): ...
    def __reduce_ex__(self, protocol): ...

class tzrange(tzrangebase):
    hasdst: bool
    def __init__(
        self,
        stdabbr: Text,
        stdoffset: Union[int, datetime.timedelta, None] = ...,
        dstabbr: Optional[Text] = ...,
        dstoffset: Union[int, datetime.timedelta, None] = ...,
        start: Optional[relativedelta] = ...,
        end: Optional[relativedelta] = ...,
    ) -> None: ...
    def transitions(self, year: int) -> Tuple[datetime.datetime, datetime.datetime]: ...
    def __eq__(self, other): ...

class tzstr(tzrange):
    hasdst: bool
    def __init__(self, s: Union[bytes, _FileObj], posix_offset: bool = ...) -> None: ...
    @classmethod
    def instance(cls, name, offset) -> tzoffset: ...

class tzical:
    def __init__(self, fileobj: _FileObj) -> None: ...
    def keys(self): ...
    def get(self, tzid: Optional[Any] = ...): ...

TZFILES: List[str]
TZPATHS: List[str]

def datetime_exists(dt: datetime.datetime, tz: Optional[datetime.tzinfo] = ...) -> bool: ...
def datetime_ambiguous(dt: datetime.datetime, tz: Optional[datetime.tzinfo] = ...) -> bool: ...
def resolve_imaginary(dt: datetime.datetime) -> datetime.datetime: ...

class _GetTZ:
    def __call__(self, name: Optional[Text] = ...) -> Optional[datetime.tzinfo]: ...
    def nocache(self, name: Optional[Text]) -> Optional[datetime.tzinfo]: ...

gettz: _GetTZ



from typing import Any

class Token:
    start_mark: Any
    end_mark: Any
    def __init__(self, start_mark, end_mark) -> None: ...

class DirectiveToken(Token):
    id: Any
    name: Any
    value: Any
    start_mark: Any
    end_mark: Any
    def __init__(self, name, value, start_mark, end_mark) -> None: ...

class DocumentStartToken(Token):
    id: Any

class DocumentEndToken(Token):
    id: Any

class StreamStartToken(Token):
    id: Any
    start_mark: Any
    end_mark: Any
    encoding: Any
    def __init__(self, start_mark=..., end_mark=..., encoding=...) -> None: ...

class StreamEndToken(Token):
    id: Any

class BlockSequenceStartToken(Token):
    id: Any

class BlockMappingStartToken(Token):
    id: Any

class BlockEndToken(Token):
    id: Any

class FlowSequenceStartToken(Token):
    id: Any

class FlowMappingStartToken(Token):
    id: Any

class FlowSequenceEndToken(Token):
    id: Any

class FlowMappingEndToken(Token):
    id: Any

class KeyToken(Token):
    id: Any

class ValueToken(Token):
    id: Any

class BlockEntryToken(Token):
    id: Any

class FlowEntryToken(Token):
    id: Any

class AliasToken(Token):
    id: Any
    value: Any
    start_mark: Any
    end_mark: Any
    def __init__(self, value, start_mark, end_mark) -> None: ...

class AnchorToken(Token):
    id: Any
    value: Any
    start_mark: Any
    end_mark: Any
    def __init__(self, value, start_mark, end_mark) -> None: ...

class TagToken(Token):
    id: Any
    value: Any
    start_mark: Any
    end_mark: Any
    def __init__(self, value, start_mark, end_mark) -> None: ...

class ScalarToken(Token):
    id: Any
    value: Any
    plain: Any
    start_mark: Any
    end_mark: Any
    style: Any
    def __init__(self, value, plain, start_mark, end_mark, style=...) -> None: ...



from typing import Any, List, Optional, Tuple, Union

from cryptography.hazmat.primitives.asymmetric.dsa import DSAPrivateKeyWithSerialization
from cryptography.hazmat.primitives.asymmetric.ec import EllipticCurvePrivateKeyWithSerialization
from cryptography.hazmat.primitives.asymmetric.rsa import RSAPrivateKeyWithSerialization
from cryptography.hazmat.primitives.serialization import KeySerializationEncryption
from cryptography.x509 import Certificate

def load_key_and_certificates(
    data: bytes, password: Optional[bytes], backend: Optional[Any] = ...
) -> Tuple[Optional[Any], Optional[Certificate], List[Certificate]]: ...
def serialize_key_and_certificates(
    name: bytes,
    key: Union[RSAPrivateKeyWithSerialization, EllipticCurvePrivateKeyWithSerialization, DSAPrivateKeyWithSerialization],
    cert: Optional[Certificate],
    cas: Optional[List[Certificate]],
    enc: KeySerializationEncryption,
) -> bytes: ...



from typing import (
    Any,
    Callable,
    Iterable,
    Iterator,
    List,
    Mapping as Mapping,
    MutableMapping as MutableMapping,
    Optional,
    Sequence,
    TypeVar,
    Union,
    overload,
)

from google.protobuf.descriptor import Descriptor
from google.protobuf.internal.message_listener import MessageListener
from google.protobuf.internal.python_message import GeneratedProtocolMessageType

_T = TypeVar("_T")
_K = TypeVar("_K")
_V = TypeVar("_V")
_M = TypeVar("_M")

class BaseContainer(Sequence[_T]):
    def __init__(self, message_listener: MessageListener) -> None: ...
    def __len__(self) -> int: ...
    def __ne__(self, other: object) -> bool: ...
    def __hash__(self) -> int: ...
    def __repr__(self) -> str: ...
    def sort(self, *, key: Optional[Callable[[_T], Any]] = ..., reverse: bool = ...) -> None: ...
    @overload
    def __getitem__(self, key: int) -> _T: ...
    @overload
    def __getitem__(self, key: slice) -> List[_T]: ...

class RepeatedScalarFieldContainer(BaseContainer[_T]):
    def __init__(self, message_listener: MessageListener, message_descriptor: Descriptor) -> None: ...
    def append(self, value: _T) -> None: ...
    def insert(self, key: int, value: _T) -> None: ...
    def extend(self, elem_seq: Optional[Iterable[_T]]) -> None: ...
    def MergeFrom(self: _M, other: _M) -> None: ...
    def remove(self, elem: _T) -> None: ...
    def pop(self, key: int = ...) -> _T: ...
    @overload
    def __setitem__(self, key: int, value: _T) -> None: ...
    @overload
    def __setitem__(self, key: slice, value: Iterable[_T]) -> None: ...
    def __getslice__(self, start: int, stop: int) -> List[_T]: ...
    def __setslice__(self, start: int, stop: int, values: Iterable[_T]) -> None: ...
    def __delitem__(self, key: Union[int, slice]) -> None: ...
    def __delslice__(self, start: int, stop: int) -> None: ...
    def __eq__(self, other: object) -> bool: ...

class RepeatedCompositeFieldContainer(BaseContainer[_T]):
    def __init__(self, message_listener: MessageListener, type_checker: Any) -> None: ...
    def add(self, **kwargs: Any) -> _T: ...
    def append(self, value: _T) -> None: ...
    def insert(self, key: int, value: _T) -> None: ...
    def extend(self, elem_seq: Iterable[_T]) -> None: ...
    def MergeFrom(self: _M, other: _M) -> None: ...
    def remove(self, elem: _T) -> None: ...
    def pop(self, key: int = ...) -> _T: ...
    def __getslice__(self, start: int, stop: int) -> List[_T]: ...
    def __delitem__(self, key: Union[int, slice]) -> None: ...
    def __delslice__(self, start: int, stop: int) -> None: ...
    def __eq__(self, other: object) -> bool: ...

class ScalarMap(MutableMapping[_K, _V]):
    def __setitem__(self, k: _K, v: _V) -> None: ...
    def __delitem__(self, v: _K) -> None: ...
    def __getitem__(self, k: _K) -> _V: ...
    def __len__(self) -> int: ...
    def __iter__(self) -> Iterator[_K]: ...
    def __eq__(self, other: object) -> bool: ...
    def MergeFrom(self: _M, other: _M): ...
    def InvalidateIterators(self) -> None: ...
    def GetEntryClass(self) -> GeneratedProtocolMessageType: ...

class MessageMap(MutableMapping[_K, _V]):
    def __setitem__(self, k: _K, v: _V) -> None: ...
    def __delitem__(self, v: _K) -> None: ...
    def __getitem__(self, k: _K) -> _V: ...
    def __len__(self) -> int: ...
    def __iter__(self) -> Iterator[_K]: ...
    def __eq__(self, other: object) -> bool: ...
    def get_or_create(self, key: _K) -> _V: ...
    def MergeFrom(self: _M, other: _M): ...
    def InvalidateIterators(self) -> None: ...
    def GetEntryClass(self) -> GeneratedProtocolMessageType: ...



from typing import Dict, List, Mapping, Optional, Sequence, Tuple, Union

_Cap = Dict[str, Union[str, int]]

def findmatch(
    caps: Mapping[str, List[_Cap]], MIMEtype: str, key: str = ..., filename: str = ..., plist: Sequence[str] = ...
) -> Tuple[Optional[str], Optional[_Cap]]: ...
def getcaps() -> Dict[str, List[_Cap]]: ...



# Stubs for six.moves
#
# Note: Commented out items means they weren't implemented at the time.
# Uncomment them when the modules have been added to the typeshed.
import __builtin__
import itertools
import os
import pipes
from __builtin__ import intern as intern, reduce as reduce, xrange as xrange
from cStringIO import StringIO as _cStringIO
from StringIO import StringIO as StringIO
from UserDict import UserDict as UserDict
from UserList import UserList as UserList
from UserString import UserString as UserString

# import Tkinter as tkinter
# import Dialog as tkinter_dialog
# import FileDialog as tkinter_filedialog
# import ScrolledText as tkinter_scrolledtext
# import SimpleDialog as tkinter_simpledialog
# import Tix as tkinter_tix
# import ttk as tkinter_ttk
# import Tkconstants as tkinter_constants
# import Tkdnd as tkinter_dnd
# import tkColorChooser as tkinter_colorchooser
# import tkCommonDialog as tkinter_commondialog
# import tkFileDialog as tkinter_tkfiledialog
# import tkFont as tkinter_font
# import tkMessageBox as tkinter_messagebox
# import tkSimpleDialog as tkinter_tksimpledialog
# import email.MIMEBase as email_mime_base
# import email.MIMEMultipart as email_mime_multipart
# import email.MIMENonMultipart as email_mime_nonmultipart
# import copy_reg as copyreg
# import gdbm as dbm_gnu
from . import (
    BaseHTTPServer,
    CGIHTTPServer,
    SimpleHTTPServer,
    _dummy_thread,
    _thread,
    configparser,
    cPickle,
    email_mime_text,
    html_entities,
    html_parser,
    http_client,
    http_cookiejar,
    http_cookies,
    queue,
    reprlib,
    socketserver,
    urllib,
    urllib_error,
    urllib_parse,
    urllib_robotparser,
    xmlrpc_client,
)

# import SimpleXMLRPCServer as xmlrpc_server

builtins = __builtin__
input = __builtin__.raw_input
reload_module = __builtin__.reload
range = __builtin__.xrange

cStringIO = _cStringIO

filter = itertools.ifilter
filterfalse = itertools.ifilterfalse
map = itertools.imap
zip = itertools.izip
zip_longest = itertools.izip_longest

getcwdb = os.getcwd
getcwd = os.getcwdu

shlex_quote = pipes.quote



import io
from typing import Any

from . import _collections, exceptions
from .connection import BaseSSLError as BaseSSLError, HTTPException as HTTPException
from .util import response

HTTPHeaderDict = _collections.HTTPHeaderDict
ProtocolError = exceptions.ProtocolError
DecodeError = exceptions.DecodeError
ReadTimeoutError = exceptions.ReadTimeoutError
binary_type = bytes  # six.binary_type
PY3 = True  # six.PY3
is_fp_closed = response.is_fp_closed

class DeflateDecoder:
    def __init__(self) -> None: ...
    def __getattr__(self, name): ...
    def decompress(self, data): ...

class GzipDecoder:
    def __init__(self) -> None: ...
    def __getattr__(self, name): ...
    def decompress(self, data): ...

class HTTPResponse(io.IOBase):
    CONTENT_DECODERS: Any
    REDIRECT_STATUSES: Any
    headers: Any
    status: Any
    version: Any
    reason: Any
    strict: Any
    decode_content: Any
    def __init__(
        self,
        body=...,
        headers=...,
        status=...,
        version=...,
        reason=...,
        strict=...,
        preload_content=...,
        decode_content=...,
        original_response=...,
        pool=...,
        connection=...,
    ) -> None: ...
    def get_redirect_location(self): ...
    def release_conn(self): ...
    @property
    def data(self): ...
    def tell(self): ...
    def read(self, amt=..., decode_content=..., cache_content=...): ...
    def stream(self, amt=..., decode_content=...): ...
    @classmethod
    def from_httplib(cls, r, **response_kw): ...
    def getheaders(self): ...
    def getheader(self, name, default=...): ...
    def close(self): ...
    @property
    def closed(self): ...
    def fileno(self): ...
    def flush(self): ...
    def readable(self): ...
    def readinto(self, b): ...



from typing import Any, Callable, ContextManager, MutableMapping, Optional, TypeVar

_KT = TypeVar("_KT")
_VT = TypeVar("_VT")
_T = TypeVar("_T", bound=Callable[..., Any])
_T_co = TypeVar("_T_co", covariant=True)
_T_self = TypeVar("_T_self")

def cached(
    cache: Optional[MutableMapping[_KT, _VT]], key: Callable[..., _KT] = ..., lock: Optional[ContextManager[_T_co]] = ...
) -> Callable[[_T], _T]: ...
def cachedmethod(
    cache: Callable[[_T_self], Optional[MutableMapping[_KT, _VT]]],
    key: Callable[..., _KT] = ...,
    lock: Optional[ContextManager[_T_co]] = ...,
) -> Callable[[_T], _T]: ...



from typing import Any, Mapping, Optional, Sequence, Text

from geoip2 import records
from geoip2.mixins import SimpleEquality

_Locales = Optional[Sequence[Text]]
_RawResponse = Mapping[Text, Mapping[Text, Any]]

class Country(SimpleEquality):
    continent: records.Continent
    country: records.Country
    registered_country: records.Country
    represented_country: records.RepresentedCountry
    maxmind: records.MaxMind
    traits: records.Traits
    raw: _RawResponse
    def __init__(self, raw_response: _RawResponse, locales: _Locales = ...) -> None: ...

class City(Country):
    city: records.City
    location: records.Location
    postal: records.Postal
    subdivisions: records.Subdivisions
    def __init__(self, raw_response: _RawResponse, locales: _Locales = ...) -> None: ...

class Insights(City): ...
class Enterprise(City): ...
class SimpleModel(SimpleEquality): ...

class AnonymousIP(SimpleModel):
    is_anonymous: bool
    is_anonymous_vpn: bool
    is_hosting_provider: bool
    is_public_proxy: bool
    is_tor_exit_node: bool
    ip_address: Optional[Text]
    raw: _RawResponse
    def __init__(self, raw: _RawResponse) -> None: ...

class ASN(SimpleModel):
    autonomous_system_number: Optional[int]
    autonomous_system_organization: Optional[Text]
    ip_address: Optional[Text]
    raw: _RawResponse
    def __init__(self, raw: _RawResponse) -> None: ...

class ConnectionType(SimpleModel):
    connection_type: Optional[Text]
    ip_address: Optional[Text]
    raw: _RawResponse
    def __init__(self, raw: _RawResponse) -> None: ...

class Domain(SimpleModel):
    domain: Optional[Text]
    ip_address: Optional[Text]
    raw: Optional[Text]
    def __init__(self, raw: _RawResponse) -> None: ...

class ISP(ASN):
    isp: Optional[Text]
    organization: Optional[Text]
    def __init__(self, raw: _RawResponse) -> None: ...



import sys
from types import ModuleType, TracebackType
from typing import Any, List, Optional, TextIO, Type, Union, overload
from typing_extensions import Literal

from _warnings import warn as warn, warn_explicit as warn_explicit

def showwarning(
    message: Union[Warning, str],
    category: Type[Warning],
    filename: str,
    lineno: int,
    file: Optional[TextIO] = ...,
    line: Optional[str] = ...,
) -> None: ...
def formatwarning(
    message: Union[Warning, str], category: Type[Warning], filename: str, lineno: int, line: Optional[str] = ...
) -> str: ...
def filterwarnings(
    action: str, message: str = ..., category: Type[Warning] = ..., module: str = ..., lineno: int = ..., append: bool = ...
) -> None: ...
def simplefilter(action: str, category: Type[Warning] = ..., lineno: int = ..., append: bool = ...) -> None: ...
def resetwarnings() -> None: ...

class _OptionError(Exception): ...

class WarningMessage:
    message: Union[Warning, str]
    category: Type[Warning]
    filename: str
    lineno: int
    file: Optional[TextIO]
    line: Optional[str]
    if sys.version_info >= (3, 6):
        source: Optional[Any]
        def __init__(
            self,
            message: Union[Warning, str],
            category: Type[Warning],
            filename: str,
            lineno: int,
            file: Optional[TextIO] = ...,
            line: Optional[str] = ...,
            source: Optional[Any] = ...,
        ) -> None: ...
    else:
        def __init__(
            self,
            message: Union[Warning, str],
            category: Type[Warning],
            filename: str,
            lineno: int,
            file: Optional[TextIO] = ...,
            line: Optional[str] = ...,
        ) -> None: ...

class catch_warnings:
    @overload
    def __new__(cls, *, record: Literal[False] = ..., module: Optional[ModuleType] = ...) -> _catch_warnings_without_records: ...
    @overload
    def __new__(cls, *, record: Literal[True], module: Optional[ModuleType] = ...) -> _catch_warnings_with_records: ...
    @overload
    def __new__(cls, *, record: bool, module: Optional[ModuleType] = ...) -> catch_warnings: ...
    def __enter__(self) -> Optional[List[WarningMessage]]: ...
    def __exit__(
        self, exc_type: Optional[Type[BaseException]], exc_val: Optional[BaseException], exc_tb: Optional[TracebackType]
    ) -> None: ...

class _catch_warnings_without_records(catch_warnings):
    def __enter__(self) -> None: ...

class _catch_warnings_with_records(catch_warnings):
    def __enter__(self) -> List[WarningMessage]: ...



from typing import Dict, List, TypeVar

MAGIC: int
MAXREPEAT: int

class error(Exception): ...

FAILURE: str
SUCCESS: str
ANY: str
ANY_ALL: str
ASSERT: str
ASSERT_NOT: str
AT: str
BIGCHARSET: str
BRANCH: str
CALL: str
CATEGORY: str
CHARSET: str
GROUPREF: str
GROUPREF_IGNORE: str
GROUPREF_EXISTS: str
IN: str
IN_IGNORE: str
INFO: str
JUMP: str
LITERAL: str
LITERAL_IGNORE: str
MARK: str
MAX_REPEAT: str
MAX_UNTIL: str
MIN_REPEAT: str
MIN_UNTIL: str
NEGATE: str
NOT_LITERAL: str
NOT_LITERAL_IGNORE: str
RANGE: str
REPEAT: str
REPEAT_ONE: str
SUBPATTERN: str
MIN_REPEAT_ONE: str
AT_BEGINNING: str
AT_BEGINNING_LINE: str
AT_BEGINNING_STRING: str
AT_BOUNDARY: str
AT_NON_BOUNDARY: str
AT_END: str
AT_END_LINE: str
AT_END_STRING: str
AT_LOC_BOUNDARY: str
AT_LOC_NON_BOUNDARY: str
AT_UNI_BOUNDARY: str
AT_UNI_NON_BOUNDARY: str
CATEGORY_DIGIT: str
CATEGORY_NOT_DIGIT: str
CATEGORY_SPACE: str
CATEGORY_NOT_SPACE: str
CATEGORY_WORD: str
CATEGORY_NOT_WORD: str
CATEGORY_LINEBREAK: str
CATEGORY_NOT_LINEBREAK: str
CATEGORY_LOC_WORD: str
CATEGORY_LOC_NOT_WORD: str
CATEGORY_UNI_DIGIT: str
CATEGORY_UNI_NOT_DIGIT: str
CATEGORY_UNI_SPACE: str
CATEGORY_UNI_NOT_SPACE: str
CATEGORY_UNI_WORD: str
CATEGORY_UNI_NOT_WORD: str
CATEGORY_UNI_LINEBREAK: str
CATEGORY_UNI_NOT_LINEBREAK: str

_T = TypeVar("_T")

def makedict(list: List[_T]) -> Dict[_T, int]: ...

OP_IGNORE: Dict[str, str]
AT_MULTILINE: Dict[str, str]
AT_LOCALE: Dict[str, str]
AT_UNICODE: Dict[str, str]
CH_LOCALE: Dict[str, str]
CH_UNICODE: Dict[str, str]
SRE_FLAG_TEMPLATE: int
SRE_FLAG_IGNORECASE: int
SRE_FLAG_LOCALE: int
SRE_FLAG_MULTILINE: int
SRE_FLAG_DOTALL: int
SRE_FLAG_UNICODE: int
SRE_FLAG_VERBOSE: int
SRE_FLAG_DEBUG: int
SRE_INFO_PREFIX: int
SRE_INFO_LITERAL: int
SRE_INFO_CHARSET: int



from typing import Any, List, Mapping, Optional, Text, Tuple, Type, Union

ssl_available: Any
hiredis_version: Any
HIREDIS_SUPPORTS_CALLABLE_ERRORS: Any
HIREDIS_SUPPORTS_BYTE_BUFFER: Any
msg: Any
HIREDIS_USE_BYTE_BUFFER: Any
SYM_STAR: Any
SYM_DOLLAR: Any
SYM_CRLF: Any
SYM_EMPTY: Any
SERVER_CLOSED_CONNECTION_ERROR: Any

class BaseParser:
    EXCEPTION_CLASSES: Any
    def parse_error(self, response): ...

class SocketBuffer:
    socket_read_size: Any
    bytes_written: Any
    bytes_read: Any
    def __init__(self, socket, socket_read_size, socket_timeout) -> None: ...
    @property
    def length(self): ...
    def read(self, length): ...
    def readline(self): ...
    def purge(self): ...
    def close(self): ...
    def can_read(self, timeout): ...

class PythonParser(BaseParser):
    encoding: Any
    socket_read_size: Any
    def __init__(self, socket_read_size) -> None: ...
    def __del__(self): ...
    def on_connect(self, connection): ...
    def on_disconnect(self): ...
    def can_read(self, timeout): ...
    def read_response(self): ...

class HiredisParser(BaseParser):
    socket_read_size: Any
    def __init__(self, socket_read_size) -> None: ...
    def __del__(self): ...
    def on_connect(self, connection): ...
    def on_disconnect(self): ...
    def can_read(self, timeout): ...
    def read_from_socket(self, timeout=..., raise_on_timeout: bool = ...) -> bool: ...
    def read_response(self): ...

DefaultParser: Any

class Encoder:
    def __init__(self, encoding, encoding_errors, decode_responses: bool) -> None: ...
    def encode(self, value: Union[Text, bytes, memoryview, bool, float]) -> bytes: ...
    def decode(self, value: Union[Text, bytes, memoryview], force: bool = ...) -> Text: ...

class Connection:
    description_format: Any
    pid: Any
    host: Any
    port: Any
    db: Any
    password: Any
    socket_timeout: Any
    socket_connect_timeout: Any
    socket_keepalive: Any
    socket_keepalive_options: Any
    retry_on_timeout: Any
    encoding: Any
    encoding_errors: Any
    decode_responses: Any
    def __init__(
        self,
        host: Text = ...,
        port: int = ...,
        db: int = ...,
        password: Optional[Text] = ...,
        socket_timeout: Optional[float] = ...,
        socket_connect_timeout: Optional[float] = ...,
        socket_keepalive: bool = ...,
        socket_keepalive_options: Optional[Mapping[str, Union[int, str]]] = ...,
        socket_type: int = ...,
        retry_on_timeout: bool = ...,
        encoding: Text = ...,
        encoding_errors: Text = ...,
        decode_responses: bool = ...,
        parser_class: Type[BaseParser] = ...,
        socket_read_size: int = ...,
        health_check_interval: int = ...,
        client_name: Optional[Text] = ...,
        username: Optional[Text] = ...,
    ) -> None: ...
    def __del__(self): ...
    def register_connect_callback(self, callback): ...
    def clear_connect_callbacks(self): ...
    def connect(self): ...
    def on_connect(self): ...
    def disconnect(self): ...
    def check_health(self) -> None: ...
    def send_packed_command(self, command, check_health: bool = ...): ...
    def send_command(self, *args): ...
    def can_read(self, timeout=...): ...
    def read_response(self): ...
    def pack_command(self, *args): ...
    def pack_commands(self, commands): ...
    def repr_pieces(self) -> List[Tuple[Text, Text]]: ...

class SSLConnection(Connection):
    description_format: Any
    keyfile: Any
    certfile: Any
    cert_reqs: Any
    ca_certs: Any
    def __init__(
        self, ssl_keyfile=..., ssl_certfile=..., ssl_cert_reqs=..., ssl_ca_certs=..., ssl_check_hostname: bool = ..., **kwargs
    ) -> None: ...

class UnixDomainSocketConnection(Connection):
    description_format: Any
    pid: Any
    path: Any
    db: Any
    password: Any
    socket_timeout: Any
    retry_on_timeout: Any
    encoding: Any
    encoding_errors: Any
    decode_responses: Any
    def __init__(
        self,
        path=...,
        db: int = ...,
        username=...,
        password=...,
        socket_timeout=...,
        encoding=...,
        encoding_errors=...,
        decode_responses=...,
        retry_on_timeout=...,
        parser_class=...,
        socket_read_size: int = ...,
        health_check_interval: int = ...,
        client_name=...,
    ) -> None: ...
    def repr_pieces(self) -> List[Tuple[Text, Text]]: ...

def to_bool(value: object) -> bool: ...

class ConnectionPool:
    @classmethod
    def from_url(cls, url: Text, db: Optional[int] = ..., decode_components: bool = ..., **kwargs) -> ConnectionPool: ...
    connection_class: Any
    connection_kwargs: Any
    max_connections: Any
    def __init__(self, connection_class=..., max_connections=..., **connection_kwargs) -> None: ...
    pid: Any
    def reset(self): ...
    def get_connection(self, command_name, *keys, **options): ...
    def make_connection(self): ...
    def release(self, connection): ...
    def disconnect(self, inuse_connections: bool = ...): ...
    def get_encoder(self) -> Encoder: ...
    def owns_connection(self, connection: Connection) -> bool: ...

class BlockingConnectionPool(ConnectionPool):
    queue_class: Any
    timeout: Any
    def __init__(self, max_connections=..., timeout=..., connection_class=..., queue_class=..., **connection_kwargs) -> None: ...
    pid: Any
    pool: Any
    def reset(self): ...
    def make_connection(self): ...
    def get_connection(self, command_name, *keys, **options): ...
    def release(self, connection): ...
    def disconnect(self): ...



import sys
from hashlib import _Hash
from typing import Any, ClassVar, Dict, Generic, Optional, Set, TypeVar, Union

from cryptography.hazmat.primitives import hashes
from cryptography.hazmat.primitives.asymmetric.ec import (
    EllipticCurvePrivateKey,
    EllipticCurvePrivateKeyWithSerialization,
    EllipticCurvePrivateNumbers,
    EllipticCurvePublicKey,
    EllipticCurvePublicKeyWithSerialization,
    EllipticCurvePublicNumbers,
)
from cryptography.hazmat.primitives.asymmetric.ed25519 import Ed25519PrivateKey, Ed25519PublicKey
from cryptography.hazmat.primitives.asymmetric.rsa import (
    RSAPrivateKey,
    RSAPrivateKeyWithSerialization,
    RSAPrivateNumbers,
    RSAPublicKey,
    RSAPublicKeyWithSerialization,
    RSAPublicNumbers,
)
from cryptography.hazmat.primitives.asymmetric.utils import Prehashed
from cryptography.hazmat.primitives.hashes import HashAlgorithm

requires_cryptography = Set[str]

def get_default_algorithms() -> Dict[str, Algorithm[Any]]: ...

_K = TypeVar("_K")

class Algorithm(Generic[_K]):
    def prepare_key(self, key: _K) -> _K: ...
    def sign(self, msg: bytes, key: _K) -> bytes: ...
    def verify(self, msg: bytes, key: _K, sig: bytes) -> bool: ...
    @staticmethod
    def to_jwk(key_obj: _K) -> str: ...
    @staticmethod
    def from_jwk(jwk: str) -> _K: ...

class NoneAlgorithm(Algorithm[None]):
    def prepare_key(self, key: Optional[str]) -> None: ...

class _HashAlg:
    def __call__(self, arg: Union[bytes, bytearray, memoryview] = ...) -> _Hash: ...

class HMACAlgorithm(Algorithm[bytes]):
    SHA256: ClassVar[_HashAlg]
    SHA384: ClassVar[_HashAlg]
    SHA512: ClassVar[_HashAlg]
    hash_alg: _HashAlg
    def __init__(self, hash_alg: _HashAlg) -> None: ...
    def prepare_key(self, key: Union[str, bytes]) -> bytes: ...
    @staticmethod
    def to_jwk(key_obj: Union[str, bytes]) -> str: ...
    @staticmethod
    def from_jwk(jwk: Union[str, bytes]) -> bytes: ...

# Only defined if cryptography is installed.
class RSAAlgorithm(Algorithm[Any]):
    SHA256: ClassVar[hashes.SHA256]
    SHA384: ClassVar[hashes.SHA384]
    SHA512: ClassVar[hashes.SHA512]
    hash_alg: Union[HashAlgorithm, Prehashed]
    def __init__(self, hash_alg: Union[HashAlgorithm, Prehashed]) -> None: ...
    def prepare_key(self, key: Union[bytes, str, RSAPrivateKey, RSAPublicKey]) -> Union[RSAPrivateKey, RSAPublicKey]: ...
    @staticmethod
    def from_jwk(jwk: Union[str, bytes, Dict[str, Any]]) -> Union[RSAPrivateKey, RSAPublicKey]: ...
    def sign(self, msg: bytes, key: RSAPrivateKey) -> bytes: ...
    def verify(self, msg: bytes, key: RSAPublicKey, sig: bytes) -> bool: ...

# Only defined if cryptography is installed.
class ECAlgorithm(Algorithm[Any]):
    SHA256: ClassVar[hashes.SHA256]
    SHA384: ClassVar[hashes.SHA384]
    SHA512: ClassVar[hashes.SHA512]
    hash_alg: Union[HashAlgorithm, Prehashed]
    def __init__(self, hash_alg: Union[HashAlgorithm, Prehashed]) -> None: ...
    def prepare_key(
        self, key: Union[bytes, str, EllipticCurvePrivateKey, EllipticCurvePublicKey]
    ) -> Union[EllipticCurvePrivateKey, EllipticCurvePublicKey]: ...
    @staticmethod
    def to_jwk(key_obj: Union[EllipticCurvePrivateKeyWithSerialization, EllipticCurvePublicKeyWithSerialization]) -> str: ...
    @staticmethod
    def from_jwk(jwk: Union[str, bytes]) -> Union[EllipticCurvePrivateKey, EllipticCurvePublicKey]: ...
    def sign(self, msg: bytes, key: EllipticCurvePrivateKey) -> bytes: ...
    def verify(self, msg: bytes, key: EllipticCurvePublicKey, sig: bytes) -> bool: ...

# Only defined if cryptography is installed. Types should be tightened when
# cryptography gets type hints.
# See https://github.com/python/typeshed/issues/2542
class RSAPSSAlgorithm(RSAAlgorithm):
    def sign(self, msg: bytes, key: Any) -> bytes: ...
    def verify(self, msg: bytes, key: Any, sig: bytes) -> bool: ...

# Only defined if cryptography is installed.
class Ed25519Algorithm(Algorithm[Any]):
    def __init__(self, **kwargs: Any) -> None: ...
    def prepare_key(self, key: Union[str, bytes, Ed25519PrivateKey, Ed25519PublicKey]) -> Any: ...
    def sign(self, msg: Union[str, bytes], key: Ed25519PrivateKey) -> bytes: ...
    def verify(self, msg: Union[str, bytes], key: Ed25519PublicKey, sig: Union[str, bytes]) -> bool: ...



from .messages import (
    CheckMessage as CheckMessage,
    Debug as Debug,
    Info as Info,
    Warning as Warning,
    Error as Error,
    Critical as Critical,
    DEBUG as DEBUG,
    INFO as INFO,
    WARNING as WARNING,
    ERROR as ERROR,
    CRITICAL as CRITICAL,
)

from .registry import register as register, run_checks as run_checks, tag_exists as tag_exists, Tags as Tags

from . import model_checks as model_checks



import os

from parso import file_io


class AbstractFolderIO:
    def __init__(self, path):
        self.path = path

    def get_base_name(self):
        raise NotImplementedError

    def list(self):
        raise NotImplementedError

    def get_file_io(self, name):
        raise NotImplementedError

    def get_parent_folder(self):
        raise NotImplementedError

    def __repr__(self):
        return '<%s: %s>' % (self.__class__.__name__, self.path)


class FolderIO(AbstractFolderIO):
    def get_base_name(self):
        return os.path.basename(self.path)

    def list(self):
        return os.listdir(self.path)

    def get_file_io(self, name):
        return FileIO(os.path.join(self.path, name))

    def get_parent_folder(self):
        return FolderIO(os.path.dirname(self.path))

    def walk(self):
        for root, dirs, files in os.walk(self.path):
            root_folder_io = FolderIO(root)
            original_folder_ios = [FolderIO(os.path.join(root, d)) for d in dirs]
            modified_folder_ios = list(original_folder_ios)
            yield (
                root_folder_io,
                modified_folder_ios,
                [FileIO(os.path.join(root, f)) for f in files],
            )
            modified_iterator = iter(reversed(modified_folder_ios))
            current = next(modified_iterator, None)
            i = len(original_folder_ios)
            for folder_io in reversed(original_folder_ios):
                i -= 1   # Basically enumerate but reversed
                if current is folder_io:
                    current = next(modified_iterator, None)
                else:
                    del dirs[i]


class FileIOFolderMixin:
    def get_parent_folder(self):
        return FolderIO(os.path.dirname(self.path))


class ZipFileIO(file_io.KnownContentFileIO, FileIOFolderMixin):
    """For .zip and .egg archives"""
    def __init__(self, path, code, zip_path):
        super().__init__(path, code)
        self._zip_path = zip_path

    def get_last_modified(self):
        try:
            return os.path.getmtime(self._zip_path)
        except (FileNotFoundError, PermissionError, NotADirectoryError):
            return None


class FileIO(file_io.FileIO, FileIOFolderMixin):
    pass


class KnownContentFileIO(file_io.KnownContentFileIO, FileIOFolderMixin):
    pass



import threading
from abc import abstractmethod
from logging import Logger
from types import TracebackType
from typing import Any, Callable, Container, Generic, Iterable, Iterator, List, Optional, Protocol, Set, Tuple, TypeVar

FIRST_COMPLETED: str
FIRST_EXCEPTION: str
ALL_COMPLETED: str
PENDING: str
RUNNING: str
CANCELLED: str
CANCELLED_AND_NOTIFIED: str
FINISHED: str
LOGGER: Logger

class Error(Exception): ...
class CancelledError(Error): ...
class TimeoutError(Error): ...

_T = TypeVar("_T")

_T_co = TypeVar("_T_co", covariant=True)

# Copied over Collection implementation as it does not exist in Python 2 and <3.6.
# Also to solve pytype issues with _Collection.
class _Collection(Iterable[_T_co], Container[_T_co], Protocol[_T_co]):
    # Implement Sized (but don't have it as a base class).
    @abstractmethod
    def __len__(self) -> int: ...

class Future(Generic[_T]):
    def __init__(self) -> None: ...
    def cancel(self) -> bool: ...
    def cancelled(self) -> bool: ...
    def running(self) -> bool: ...
    def done(self) -> bool: ...
    def add_done_callback(self, fn: Callable[[Future[_T]], Any]) -> None: ...
    def result(self, timeout: Optional[float] = ...) -> _T: ...
    def set_running_or_notify_cancel(self) -> bool: ...
    def set_result(self, result: _T) -> None: ...
    def exception(self, timeout: Optional[float] = ...) -> Any: ...
    def exception_info(self, timeout: Optional[float] = ...) -> Tuple[Any, Optional[TracebackType]]: ...
    def set_exception(self, exception: Any) -> None: ...
    def set_exception_info(self, exception: Any, traceback: Optional[TracebackType]) -> None: ...

class Executor:
    def submit(self, fn: Callable[..., _T], *args: Any, **kwargs: Any) -> Future[_T]: ...
    def map(self, func: Callable[..., _T], *iterables: Iterable[Any], timeout: Optional[float] = ...) -> Iterator[_T]: ...
    def shutdown(self, wait: bool = ...) -> None: ...
    def __enter__(self: _T) -> _T: ...
    def __exit__(self, exc_type: Any, exc_val: Any, exc_tb: Any) -> Optional[bool]: ...

def as_completed(fs: Iterable[Future[_T]], timeout: Optional[float] = ...) -> Iterator[Future[_T]]: ...
def wait(
    fs: _Collection[Future[_T]], timeout: Optional[float] = ..., return_when: str = ...
) -> Tuple[Set[Future[_T]], Set[Future[_T]]]: ...

class _Waiter:
    event: threading.Event
    finished_futures: List[Future[Any]]
    def __init__(self) -> None: ...
    def add_result(self, future: Future[Any]) -> None: ...
    def add_exception(self, future: Future[Any]) -> None: ...
    def add_cancelled(self, future: Future[Any]) -> None: ...

class _AsCompletedWaiter(_Waiter):
    lock: threading.Lock
    def __init__(self) -> None: ...
    def add_result(self, future: Future[Any]) -> None: ...
    def add_exception(self, future: Future[Any]) -> None: ...
    def add_cancelled(self, future: Future[Any]) -> None: ...

class _FirstCompletedWaiter(_Waiter):
    def add_result(self, future: Future[Any]) -> None: ...
    def add_exception(self, future: Future[Any]) -> None: ...
    def add_cancelled(self, future: Future[Any]) -> None: ...

class _AllCompletedWaiter(_Waiter):
    num_pending_calls: int
    stop_on_exception: bool
    lock: threading.Lock
    def __init__(self, num_pending_calls: int, stop_on_exception: bool) -> None: ...
    def add_result(self, future: Future[Any]) -> None: ...
    def add_exception(self, future: Future[Any]) -> None: ...
    def add_cancelled(self, future: Future[Any]) -> None: ...

class _AcquireFutures:
    futures: Iterable[Future[Any]]
    def __init__(self, futures: Iterable[Future[Any]]) -> None: ...
    def __enter__(self) -> None: ...
    def __exit__(self, *args: Any) -> None: ...



from django.db.models import *

from .fields import (
    GeometryField as GeometryField,
    LineStringField as LineStringField,
    MultiLineStringField as MultiLineStringField,
    MultiPointField as MultiPointField,
    MultiPolygonField as MultiPolygonField,
    PointField as PointField,
    PolygonField as PolygonField,
    GeometryCollectionField as GeometryCollectionField,
    RasterField as RasterField,
)



from json import JSONEncoder
from typing import Any, Optional, Type

from django.db.models import Field
from django.db.models.lookups import Transform
from .mixins import CheckFieldDefaultMixin

class JsonAdapter:
    encoder: Any = ...
    def __init__(self, adapted: Any, dumps: Optional[Any] = ..., encoder: Optional[Any] = ...) -> None: ...
    def dumps(self, obj: Any): ...

class JSONField(CheckFieldDefaultMixin, Field):
    empty_strings_allowed: bool = ...
    description: Any = ...
    default_error_messages: Any = ...
    encoder: Any = ...
    def __init__(
        self,
        verbose_name: Optional[str] = ...,
        name: Optional[str] = ...,
        encoder: Optional[Type[JSONEncoder]] = ...,
        **kwargs: Any
    ) -> None: ...
    def value_to_string(self, obj: Any): ...

class KeyTransform(Transform):
    operator: str = ...
    nested_operator: str = ...
    def __init__(self, key_name: str, *args: Any, **kwargs: Any) -> None: ...

class KeyTextTransform(KeyTransform): ...
